@@ -16,10 +16,10 @@
  */
 
 #include "gpu_validation/debug_printf.h"
-#include "spirv-tools/optimizer.hpp"
 #include "spirv-tools/instrument.hpp"
 #include <iostream>
 #include "generated/layer_chassis_dispatch.h"
+#include "utils/shader_utils.h"
 
 // Perform initializations that can be done at Create Device time.
 void DebugPrintf::CreateDevice(const VkDeviceCreateInfo *pCreateInfo) {
@@ -34,11 +34,11 @@ void DebugPrintf::CreateDevice(const VkDeviceCreateInfo *pCreateInfo) {
     output_buffer_size = *size_string ? atoi(size_string) : 1024;
 
     std::string verbose_string = getLayerOption("khronos_validation.printf_verbose");
-    transform(verbose_string.begin(), verbose_string.end(), verbose_string.begin(), ::tolower);
+    vvl::ToLower(verbose_string);
     verbose = !verbose_string.compare("true");
 
     std::string stdout_string = getLayerOption("khronos_validation.printf_to_stdout");
-    transform(stdout_string.begin(), stdout_string.end(), stdout_string.begin(), ::tolower);
+    vvl::ToLower(stdout_string);
     use_stdout = !stdout_string.compare("true");
     if (getenv("DEBUG_PRINTF_TO_STDOUT")) use_stdout = true;
 
@@ -129,7 +129,8 @@ bool DebugPrintf::InstrumentShader(const vvl::span<const uint32_t> &input, std::
 void DebugPrintf::PreCallRecordCreateShaderModule(VkDevice device, const VkShaderModuleCreateInfo *pCreateInfo,
                                                   const VkAllocationCallbacks *pAllocator, VkShaderModule *pShaderModule,
                                                   void *csm_state_data) {
-    create_shader_module_api_state *csm_state = reinterpret_cast<create_shader_module_api_state *>(csm_state_data);
+    ValidationStateTracker::PreCallRecordCreateShaderModule(device, pCreateInfo, pAllocator, pShaderModule, csm_state_data);
+    create_shader_module_api_state *csm_state = static_cast<create_shader_module_api_state *>(csm_state_data);
     const bool pass = InstrumentShader(vvl::make_span(pCreateInfo->pCode, pCreateInfo->codeSize / sizeof(uint32_t)),
                                        csm_state->instrumented_pgm, &csm_state->unique_shader_id);
     if (pass) {
@@ -138,6 +139,12 @@ void DebugPrintf::PreCallRecordCreateShaderModule(VkDevice device, const VkShade
     }
 }
 
+void DebugPrintf::PreCallRecordCreateShadersEXT(VkDevice device, uint32_t createInfoCount,
+                                                const VkShaderCreateInfoEXT *pCreateInfos, const VkAllocationCallbacks *pAllocator,
+                                                VkShaderEXT *pShaders, void *csm_state_data) {
+    // TODO - Add VK_EXT_shader_object support
+}
+
 vartype vartype_lookup(char intype) {
     switch (intype) {
         case 'd':
@@ -242,19 +249,15 @@ std::vector<DPFSubstring> DebugPrintf::ParseFormatString(const std::string &form
 
 std::string DebugPrintf::FindFormatString(vvl::span<const uint32_t> pgm, uint32_t string_id) {
     std::string format_string;
-    SHADER_MODULE_STATE module_state(pgm);
+    SPIRV_MODULE_STATE module_state(pgm);
     if (module_state.words_.empty()) {
         return {};
     }
-    for (const Instruction &insn : module_state.GetInstructions()) {
-        if (insn.Opcode() == spv::OpFunction) {
-            break;  // Debug Info is always before first function
-        }
-        if (insn.Opcode() == spv::OpString) {
-            if (insn.Word(1) == string_id) {
-                format_string = insn.GetAsString(2);
-                break;
-            }
+
+    for (const Instruction *insn : module_state.static_data_.debug_string_inst) {
+        if (insn->Word(1) == string_id) {
+            format_string = insn->GetAsString(2);
+            break;
         }
     }
 
@@ -326,6 +329,7 @@ void DebugPrintf::AnalyzeAndGenerateMessages(VkCommandBuffer command_buffer, VkQ
             pipeline_handle = it->second.pipeline;
             pgm = it->second.pgm;
         }
+        assert(pgm.size() != 0);
         // Search through the shader source for the printf format string for this invocation
         const auto format_string = FindFormatString(pgm, debug_record->format_string_id);
         // Break the format string into strings with 1 or 0 value
