@@ -17,15 +17,30 @@
 
 #pragma once
 
-#include "gpu_validation/gpu_utils.h"
+#include "gpu_validation/gpu_state_tracker.h"
+#include "gpu_validation/gpu_error_message.h"
+#include "gpu_validation/gv_descriptor_sets.h"
 #include "state_tracker/pipeline_state.h"
 
 class GpuAssisted;
 
+struct GpuAssistedDescSetState {
+    std::shared_ptr<gpuav_state::DescriptorSet> set_state;
+    // State that will be used by the GPU-AV shader instrumentation
+    // For update-after-bind, this will be set during queue submission
+    // Otherwise it will be set when the DescriptorSet is bound.
+    std::shared_ptr<gpuav_state::DescriptorSet::State> gpu_state;
+};
+
 struct GpuAssistedDeviceMemoryBlock {
     VkBuffer buffer;
     VmaAllocation allocation;
-    vvl::unordered_map<uint32_t, const cvdescriptorset::DescriptorBinding*> update_at_submit;
+};
+
+struct GpuAssistedInputBuffers {
+    VkBuffer address_buffer;
+    VmaAllocation address_buffer_allocation;
+    std::vector<GpuAssistedDescSetState> descriptor_set_buffers;
 };
 
 struct GpuAssistedPreDrawResources {
@@ -48,7 +63,6 @@ struct GpuAssistedPreDispatchResources {
 
 struct GpuAssistedBufferInfo {
     GpuAssistedDeviceMemoryBlock output_mem_block;
-    GpuAssistedDeviceMemoryBlock bda_input_mem_block;  // Buffer Device Address input
     GpuAssistedPreDrawResources pre_draw_resources;
     GpuAssistedPreDispatchResources pre_dispatch_resources;
     VkDescriptorSet desc_set;
@@ -56,19 +70,20 @@ struct GpuAssistedBufferInfo {
     VkPipelineBindPoint pipeline_bind_point;
     bool uses_robustness;
     CMD_TYPE cmd_type;
-    GpuAssistedBufferInfo(GpuAssistedDeviceMemoryBlock output_mem_block, GpuAssistedDeviceMemoryBlock bda_input_mem_block,
-                          GpuAssistedPreDrawResources pre_draw_resources, GpuAssistedPreDispatchResources pre_dispatch_resources,
-                          VkDescriptorSet desc_set, VkDescriptorPool desc_pool, VkPipelineBindPoint pipeline_bind_point,
-                          bool uses_robustness, CMD_TYPE cmd_type)
+    uint32_t desc_binding_index;
+    GpuAssistedBufferInfo(GpuAssistedDeviceMemoryBlock output_mem_block, GpuAssistedPreDrawResources pre_draw_resources,
+                          GpuAssistedPreDispatchResources pre_dispatch_resources, VkDescriptorSet desc_set,
+                          VkDescriptorPool desc_pool, VkPipelineBindPoint pipeline_bind_point, bool uses_robustness,
+                          CMD_TYPE cmd_type, uint32_t desc_binding_index)
         : output_mem_block(output_mem_block),
-          bda_input_mem_block(bda_input_mem_block),
           pre_draw_resources(pre_draw_resources),
           pre_dispatch_resources(pre_dispatch_resources),
           desc_set(desc_set),
           desc_pool(desc_pool),
           pipeline_bind_point(pipeline_bind_point),
           uses_robustness(uses_robustness),
-          cmd_type(cmd_type){};
+          cmd_type(cmd_type),
+          desc_binding_index(desc_binding_index){};
 };
 
 struct GpuVuid {
@@ -149,7 +164,7 @@ namespace gpuav_state {
 class CommandBuffer : public gpu_utils_state::CommandBuffer {
   public:
     std::vector<GpuAssistedBufferInfo> per_draw_buffer_list;
-    std::vector<GpuAssistedDeviceMemoryBlock> di_input_buffer_list;
+    std::vector<GpuAssistedInputBuffers> di_input_buffer_list;
     std::vector<GpuAssistedAccelerationStructureBuildValidationBufferInfo> as_validation_buffers;
     VkBuffer current_input_buffer = VK_NULL_HANDLE;
 
@@ -167,9 +182,11 @@ class CommandBuffer : public gpu_utils_state::CommandBuffer {
     void ResetCBState();
     void ProcessAccelerationStructure(VkQueue queue);
 };
+
 }  // namespace gpuav_state
 
 VALSTATETRACK_DERIVED_STATE_OBJECT(VkCommandBuffer, gpuav_state::CommandBuffer, CMD_BUFFER_STATE)
+VALSTATETRACK_DERIVED_STATE_OBJECT(VkDescriptorSet, gpuav_state::DescriptorSet, cvdescriptorset::DescriptorSet)
 
 class GpuAssisted : public GpuAssistedBase {
   public:
@@ -179,6 +196,7 @@ class GpuAssisted : public GpuAssistedBase {
         desired_features.vertexPipelineStoresAndAtomics = true;
         desired_features.fragmentStoresAndAtomics = true;
         desired_features.shaderInt64 = true;
+        force_buffer_device_address = true;
     }
 
     bool CheckForDescriptorIndexing(DeviceFeatures enabled_features) const;
@@ -200,15 +218,24 @@ class GpuAssisted : public GpuAssistedBase {
     void PreCallRecordCreateShaderModule(VkDevice device, const VkShaderModuleCreateInfo* pCreateInfo,
                                          const VkAllocationCallbacks* pAllocator, VkShaderModule* pShaderModule,
                                          void* csm_state_data) override;
+    void PreCallRecordCreateShadersEXT(VkDevice device, uint32_t createInfoCount, const VkShaderCreateInfoEXT* pCreateInfos,
+                                       const VkAllocationCallbacks* pAllocator, VkShaderEXT* pShaders,
+                                       void* csm_state_data) override;
     void AnalyzeAndGenerateMessages(VkCommandBuffer command_buffer, VkQueue queue, GpuAssistedBufferInfo& buffer_info,
-                                    uint32_t operation_index, uint32_t* const debug_output_buffer);
-
-    void SetBindingState(uint32_t* data, uint32_t index, const cvdescriptorset::DescriptorBinding* binding);
+                                    uint32_t operation_index, uint32_t* const debug_output_buffer,
+                                    const std::vector<GpuAssistedDescSetState>& descriptor_sets);
     void UpdateInstrumentationBuffer(gpuav_state::CommandBuffer* cb_node);
+    void UpdateBDABuffer(GpuAssistedDeviceMemoryBlock buffer_device_addresses);
+
+    void UpdateBoundDescriptors(VkCommandBuffer commandBuffer, VkPipelineBindPoint pipelineBindPoint);
+
     void PostCallRecordCmdBindDescriptorSets(VkCommandBuffer commandBuffer, VkPipelineBindPoint pipelineBindPoint,
                                              VkPipelineLayout layout, uint32_t firstSet, uint32_t descriptorSetCount,
                                              const VkDescriptorSet* pDescriptorSets, uint32_t dynamicOffsetCount,
                                              const uint32_t* pDynamicOffsets) override;
+    void PreCallRecordCmdPushDescriptorSetKHR(VkCommandBuffer commandBuffer, VkPipelineBindPoint pipelineBindPoint,
+                                              VkPipelineLayout layout, uint32_t set, uint32_t descriptorWriteCount,
+                                              const VkWriteDescriptorSet* pDescriptorWrites) override;
     void PreCallRecordQueueSubmit(VkQueue queue, uint32_t submitCount, const VkSubmitInfo* pSubmits, VkFence fence) override;
     void PreCallRecordQueueSubmit2KHR(VkQueue queue, uint32_t submitCount, const VkSubmitInfo2KHR* pSubmits,
                                       VkFence fence) override;
@@ -296,6 +323,9 @@ class GpuAssisted : public GpuAssistedBase {
 
     std::shared_ptr<CMD_BUFFER_STATE> CreateCmdBufferState(VkCommandBuffer cb, const VkCommandBufferAllocateInfo* create_info,
                                                            const COMMAND_POOL_STATE* pool) final;
+    std::shared_ptr<cvdescriptorset::DescriptorSet> CreateDescriptorSet(
+        VkDescriptorSet, DESCRIPTOR_POOL_STATE*, const std::shared_ptr<cvdescriptorset::DescriptorSetLayout const>& layout,
+        uint32_t variable_count) final;
 
     void DestroyBuffer(GpuAssistedBufferInfo& buffer_info);
     void DestroyBuffer(GpuAssistedAccelerationStructureBuildValidationBufferInfo& buffer_info);
@@ -305,7 +335,7 @@ class GpuAssisted : public GpuAssistedBase {
     VkPipeline GetValidationPipeline(VkRenderPass render_pass);
 
     VkBool32 shaderInt64;
-    bool buffer_oob_enabled;
+    bool validate_descriptors;
     bool validate_draw_indirect;
     bool validate_dispatch_indirect;
     bool warn_on_robust_oob;
@@ -313,7 +343,10 @@ class GpuAssisted : public GpuAssistedBase {
     GpuAssistedAccelerationStructureBuildValidationState acceleration_structure_validation_state;
     GpuAssistedPreDrawValidationState pre_draw_validation_state;
     GpuAssistedPreDispatchValidationState pre_dispatch_validation_state;
+    GpuAssistedDeviceMemoryBlock app_buffer_device_addresses{};
+    size_t app_bda_buffer_size{};
+    size_t app_bda_max_addresses{};
+    uint32_t gpuav_bda_buffer_version = 0;
 
-    bool descriptor_indexing = false;
     bool buffer_device_address;
 };
