@@ -22,6 +22,7 @@
 #include "state_tracker/query_state.h"
 #include "state_tracker/video_session_state.h"
 #include "generated/command_validation.h"
+#include "generated/dynamic_state_helper.h"
 #include "utils/hash_vk_types.h"
 #include "containers/subresource_adapter.h"
 #include "state_tracker/image_layout_map.h"
@@ -107,12 +108,12 @@ class COMMAND_POOL_STATE : public BASE_NODE {
 // Autogenerated as part of the command_validation.h codegen
 const char *CommandTypeString(CMD_TYPE type);
 
-enum CB_STATE {
-    CB_NEW,                 // Newly created CB w/o any cmds
-    CB_RECORDING,           // BeginCB has been called on this CB
-    CB_RECORDED,            // EndCB has been called on this CB
-    CB_INVALID_COMPLETE,    // had a complete recording, but was since invalidated
-    CB_INVALID_INCOMPLETE,  // fouled before recording was completed
+enum class CbState {
+    New,                // Newly created CB w/o any cmds
+    Recording,          // BeginCB has been called on this CB
+    Recorded,           // EndCB has been called on this CB
+    InvalidComplete,    // had a complete recording, but was since invalidated
+    InvalidIncomplete,  // fouled before recording was completed
 };
 
 struct BufferBinding {
@@ -139,6 +140,11 @@ struct IndexBufferBinding : BufferBinding {
     IndexBufferBinding() : BufferBinding(), index_type(static_cast<VkIndexType>(0)) {}
     IndexBufferBinding(const std::shared_ptr<BUFFER_STATE> &buffer_state_, VkDeviceSize offset_, VkIndexType index_type_)
         : BufferBinding(buffer_state_, offset_), index_type(index_type_) {}
+    // TODO - We could clean up the BufferBinding interface now we have 2 ways to bind both the Vertex and Index buffer
+    IndexBufferBinding(const std::shared_ptr<BUFFER_STATE> &buffer_state_, VkDeviceSize size_, VkDeviceSize offset_,
+                       VkIndexType index_type_)
+        : BufferBinding(buffer_state_, BUFFER_STATE::ComputeSize(buffer_state_, offset_, size_), offset_, 0U),
+          index_type(index_type_) {}
     virtual ~IndexBufferBinding() {}
 
     virtual void reset() override { *this = IndexBufferBinding(); }
@@ -173,17 +179,19 @@ class CMD_BUFFER_STATE : public REFCOUNTED_NODE {
     bool has_trace_rays_cmd;
     bool has_build_as_cmd;
 
-    CB_STATE state;               // Track cmd buffer update state
-    uint64_t command_count;       // Number of commands recorded. Currently only used with VK_KHR_performance_query
-    uint64_t submitCount;         // Number of times CB has been submitted
+    CbState state;           // Track cmd buffer update state
+    uint64_t command_count;  // Number of commands recorded. Currently only used with VK_KHR_performance_query
+    uint64_t submitCount;    // Number of times CB has been submitted
     typedef uint64_t ImageLayoutUpdateCount;
     ImageLayoutUpdateCount image_layout_change_count;  // The sequence number for changes to image layout (for cached validation)
 
-    // Dynamic State
-    CBDynamicFlags status;          // Track status of various bindings on cmd buffer
-    CBDynamicFlags static_status;   // All state bits provided by current graphics pipeline
-                                    // rather than dynamic state
-    CBDynamicFlags dynamic_status;  // dynamic state set up in pipeline
+    // Track status of all vkCmdSet* calls, if 1, means it was set
+    struct DynamicStateStatus {
+        CBDynamicFlags cb;        // for lifetime of CommandBuffer
+        CBDynamicFlags pipeline;  // for lifetime since last bound pipeline
+    } dynamic_state_status;
+
+    // These are values that are being set with vkCmdSet* tied to a command buffer
     struct DynamicStateValue {
         // VK_DYNAMIC_STATE_STENCIL_WRITE_MASK
         uint32_t write_mask_front;
@@ -192,6 +200,8 @@ class CMD_BUFFER_STATE : public REFCOUNTED_NODE {
         bool depth_write_enable;
         // VK_DYNAMIC_STATE_DEPTH_TEST_ENABLE
         bool depth_test_enable;
+        // VK_DYNAMIC_STATE_DEPTH_BOUNDS_TEST_ENABLE
+        bool depth_bounds_test_enable;
         // VK_DYNAMIC_STATE_STENCIL_TEST_ENABLE
         bool stencil_test_enable;
         // VK_DYNAMIC_STATE_STENCIL_OP
@@ -201,37 +211,108 @@ class CMD_BUFFER_STATE : public REFCOUNTED_NODE {
         VkStencilOp fail_op_back;
         VkStencilOp pass_op_back;
         VkStencilOp depth_fail_op_back;
+        // VK_DYNAMIC_STATE_CULL_MODE
+        VkCullModeFlags cull_mode;
         // VK_DYNAMIC_STATE_PRIMITIVE_TOPOLOGY
         VkPrimitiveTopology primitive_topology;
+        // VK_DYNAMIC_STATE_DISCARD_RECTANGLE_ENABLE_EXT
+        bool discard_rectangle_enable;
         // VK_DYNAMIC_STATE_DISCARD_RECTANGLE_EXT
         // maxDiscardRectangles is at max 8 on all known implementations currently
         std::bitset<32> discard_rectangles;
+        // VK_DYNAMIC_STATE_POLYGON_MODE_EXT
+        VkPolygonMode polygon_mode;
         // VK_DYNAMIC_STATE_RASTERIZATION_SAMPLES_EXT
         VkSampleCountFlagBits rasterization_samples;
         // VK_DYNAMIC_STATE_LINE_RASTERIZATION_MODE_EXT
         VkLineRasterizationModeEXT line_rasterization_mode;
         // VK_DYNAMIC_STATE_LINE_STIPPLE_ENABLE_EXT
         bool stippled_line_enable;
+        // VK_DYNAMIC_STATE_COVERAGE_TO_COLOR_ENABLE_NV
+        bool coverage_to_color_enable;
+        // VK_DYNAMIC_STATE_COVERAGE_MODULATION_MODE_NV
+        VkCoverageModulationModeNV coverage_modulation_mode;
+        // VK_DYNAMIC_STATE_COVERAGE_MODULATION_TABLE_ENABLE_NV
+        bool coverage_modulation_table_enable;
+        // VK_DYNAMIC_STATE_SHADING_RATE_IMAGE_ENABLE_NV
+        bool shading_rate_image_enable;
+        // VK_DYNAMIC_STATE_RASTERIZER_DISCARD_ENABLE
+        bool rasterizer_discard_enable;
+        // VK_DYNAMIC_STATE_DEPTH_BIAS_ENABLE
+        bool depth_bias_enable = false;
+        // VK_DYNAMIC_STATE_ALPHA_TO_COVERAGE_ENABLE_EXT
+        bool alpha_to_coverage_enable;
+        // VK_DYNAMIC_STATE_LOGIC_OP_ENABLE_EXT
+        bool logic_op_enable;
+
+        uint32_t color_write_enable_attachment_count;
 
         // maxColorAttachments is at max 8 on all known implementations currently
-        std::bitset<32> color_blend_enable_attachments;    // VK_DYNAMIC_STATE_COLOR_BLEND_ENABLE_EXT
-        std::bitset<32> color_blend_equation_attachments;  // VK_DYNAMIC_STATE_COLOR_BLEND_EQUATION_EXT
-        std::bitset<32> color_write_mask_attachments;      // VK_DYNAMIC_STATE_COLOR_WRITE_MASK_EXT
-        std::bitset<32> color_blend_advanced_attachments;  // VK_DYNAMIC_STATE_COLOR_BLEND_ADVANCED_EXT
+        std::bitset<32> color_blend_enable_attachments;              // VK_DYNAMIC_STATE_COLOR_BLEND_ENABLE_EXT
+        std::bitset<32> color_blend_enabled;                         // VK_DYNAMIC_STATE_COLOR_BLEND_ENABLE_EXT
+        std::bitset<32> color_blend_equation_attachments;            // VK_DYNAMIC_STATE_COLOR_BLEND_EQUATION_EXT
+        std::bitset<32> color_write_mask_attachments;                // VK_DYNAMIC_STATE_COLOR_WRITE_MASK_EXT
+        std::bitset<32> color_blend_advanced_attachments;            // VK_DYNAMIC_STATE_COLOR_BLEND_ADVANCED_EXT
+        std::vector<VkColorBlendEquationEXT> color_blend_equations;  // VK_DYNAMIC_STATE_COLOR_BLEND_EQUATION_EXT
+        std::vector<VkColorComponentFlags> color_write_masks;        // VK_DYNAMIC_STATE_COLOR_WRITE_MASK_EXT
 
+        // VK_DYNAMIC_STATE_VERTEX_INPUT_EXT
+        std::vector<VkVertexInputAttributeDescription2EXT> vertex_attribute_descriptions;
+
+        // VK_DYNAMIC_STATE_CONSERVATIVE_RASTERIZATION_MODE_EXT
+        VkConservativeRasterizationModeEXT conservative_rasterization_mode;
+        // VK_DYNAMIC_STATE_SAMPLE_LOCATIONS_ENABLE_EXT
+        bool sample_locations_enable;
+
+        // VK_DYNAMIC_STATE_VIEWPORT
+        std::vector<VkViewport> viewports;
+        // and VK_DYNAMIC_STATE_VIEWPORT_WITH_COUNT
+        uint32_t viewport_count;
+        // VK_DYNAMIC_STATE_SCISSOR_WITH_COUNT
+        uint32_t scissor_count;
+        // VK_DYNAMIC_STATE_VIEWPORT_W_SCALING_NV
+        std::vector<VkViewportWScalingNV> viewport_w_scalings;
+        uint32_t viewport_w_scaling_first;
+        uint32_t viewport_w_scaling_count;
+        // VK_DYNAMIC_STATE_VIEWPORT_W_SCALING_ENABLE
+        bool viewport_w_scaling_enable;
+        // VK_DYNAMIC_STATE_VIEWPORT_SHADING_RATE_PALETTE_NV
+        uint32_t shading_rate_palette_count;
+        // VK_DYNAMIC_STATE_EXCLUSIVE_SCISSOR_ENABLE_NV
+        uint32_t exclusive_scissor_enable_first;
+        uint32_t exclusive_scissor_enable_count;
+        std::vector<VkBool32> exclusive_scissor_enables;
+        // VK_DYNAMIC_STATE_EXCLUSIVE_SCISSOR_NV
+        uint32_t exclusive_scissor_first;
+        uint32_t exclusive_scissor_count;
+        std::vector<VkRect2D> exclusive_scissors;
         // When the Command Buffer resets, the value most things in this struct don't matter because if they are read without
-        // setting the state, it will fail in ValidateCBDynamicStatus() for us. Some values (ex. the bitset) are tracking in
+        // setting the state, it will fail in ValidateDynamicStateIsSet() for us. Some values (ex. the bitset) are tracking in
         // replacement for static_status/dynamic_status so this needs to reset along with those
         void reset() {
+            // There are special because the Secondary CB Inheritance is tracking these defaults
+            viewport_count = 0u;
+            scissor_count = 0u;
+
+            depth_bias_enable = false;
+
+            viewports.clear();
             discard_rectangles.reset();
             color_blend_enable_attachments.reset();
             color_blend_equation_attachments.reset();
             color_write_mask_attachments.reset();
             color_blend_advanced_attachments.reset();
+            color_blend_equations.clear();
+            color_write_masks.clear();
+            vertex_attribute_descriptions.clear();
+            viewport_w_scalings.clear();
+            exclusive_scissor_enables.clear();
+            exclusive_scissors.clear();
+
+            color_write_enable_attachment_count = 0u;
         }
     } dynamic_state_value;
 
-    std::string begin_rendering_func_name;
     // Currently storing "lastBound" objects on per-CB basis
     //  long-term may want to create caches of "lastBound" states and could have
     //  each individual CMD_NODE referencing its own "lastBound" state
@@ -267,14 +348,8 @@ class CMD_BUFFER_STATE : public REFCOUNTED_NODE {
 
     uint32_t viewportMask;
     uint32_t viewportWithCountMask;
-    uint32_t viewportWithCountCount;
     uint32_t scissorMask;
     uint32_t scissorWithCountMask;
-    uint32_t scissorWithCountCount;
-
-    // Dynamic viewports set in this command buffer; if bit j of viewportMask is set then dynamicViewports[j] is valid, but the
-    // converse need not be true.
-    std::vector<VkViewport> dynamicViewports;
 
     // Bits set when binding graphics pipeline defining corresponding static state, or executing any secondary command buffer.
     // Bits unset by calling a corresponding vkCmdSet[State] cmd.
@@ -289,17 +364,18 @@ class CMD_BUFFER_STATE : public REFCOUNTED_NODE {
 
     uint32_t initial_device_mask;
 
-    bool rasterization_disabled = false;
-
-    safe_VkRenderPassBeginInfo activeRenderPassBeginInfo;
+    // The RenderPass created from vkCmdBeginRenderPass or vkCmdBeginRendering
     std::shared_ptr<RENDER_PASS_STATE> activeRenderPass;
+    // Used for both type of renderPass
+    vvl::unordered_set<uint32_t> active_color_attachments_index;
+    uint32_t active_render_pass_device_mask;
+    // only when not using dynamic rendering
+    safe_VkRenderPassBeginInfo active_render_pass_begin_info;
     std::shared_ptr<std::vector<SUBPASS_INFO>> active_subpasses;
     std::shared_ptr<std::vector<IMAGE_VIEW_STATE *>> active_attachments;
     std::set<std::shared_ptr<IMAGE_VIEW_STATE>> attachments_view_states;
-    vvl::unordered_set<uint32_t> active_color_attachments_index;
 
     VkSubpassContents activeSubpassContents;
-    uint32_t active_render_pass_device_mask;
     uint32_t GetActiveSubpass() const { return active_subpass_; }
     void SetActiveSubpass(uint32_t subpass);
     std::optional<VkSampleCountFlagBits> GetActiveSubpassRasterizationSampleCount() const { return active_subpass_sample_count_; }
@@ -357,9 +433,6 @@ class CMD_BUFFER_STATE : public REFCOUNTED_NODE {
     std::vector<uint8_t> push_constant_data;
     PushConstantRangesId push_constant_data_ranges;
 
-    std::map<VkShaderStageFlagBits, std::vector<uint8_t>>
-        push_constant_data_update;  // vector's value is enum PushConstantByteState.
-
     // Used for Best Practices tracking
     uint32_t small_indexed_draw_call_count;
 
@@ -373,7 +446,6 @@ class CMD_BUFFER_STATE : public REFCOUNTED_NODE {
     bool conditional_rendering_active{false};
     bool conditional_rendering_inside_render_pass{false};
     uint32_t conditional_rendering_subpass{0};
-    uint32_t dynamicColorWriteEnableAttachmentCount{0};
     std::vector<VkDescriptorBufferBindingInfoEXT> descriptor_buffer_binding_info;
 
     mutable std::shared_mutex lock;
@@ -398,10 +470,11 @@ class CMD_BUFFER_STATE : public REFCOUNTED_NODE {
         auto base = std::static_pointer_cast<BASE_NODE>(child_node);
         AddChild(base);
     }
-    template <typename StateObject>
-    void AddChildren(vvl::span<std::shared_ptr<StateObject>> &child_nodes) {
+
+    void AddChildren(vvl::span<BUFFER_STATE *> &child_nodes) {
         for (auto &child_node : child_nodes) {
-            AddChild(child_node);
+            auto child_node_shared_ptr = child_node->shared_from_this();
+            AddChild(child_node_shared_ptr);
         }
     }
 
@@ -472,6 +545,7 @@ class CMD_BUFFER_STATE : public REFCOUNTED_NODE {
     void EndRenderPass(CMD_TYPE cmd_type);
 
     void BeginRendering(CMD_TYPE cmd_type, const VkRenderingInfo *pRenderingInfo);
+    void EndRendering(CMD_TYPE cmd_type);
 
     void BeginVideoCoding(const VkVideoBeginCodingInfoKHR *pBeginInfo);
     void EndVideoCoding(const VkVideoEndCodingInfoKHR *pEndCodingInfo);
@@ -498,9 +572,8 @@ class CMD_BUFFER_STATE : public REFCOUNTED_NODE {
     void UpdatePipelineState(CMD_TYPE cmd_type, const VkPipelineBindPoint bind_point);
 
     virtual void RecordCmd(CMD_TYPE cmd_type);
-    void RecordStateCmd(CMD_TYPE cmd_type, CBDynamicStatus state);
+    void RecordStateCmd(CMD_TYPE cmd_type, CBDynamicState dynamic_state);
     void RecordStateCmd(CMD_TYPE cmd_type, CBDynamicFlags const &state_bits);
-    void RecordColorWriteEnableStateCmd(CMD_TYPE cmd_type, CBDynamicStatus state, uint32_t attachment_count);
     void RecordTransferCmd(CMD_TYPE cmd_type, std::shared_ptr<BINDABLE> &&buf1, std::shared_ptr<BINDABLE> &&buf2 = nullptr);
     void RecordSetEvent(CMD_TYPE cmd_type, VkEvent event, VkPipelineStageFlags2KHR stageMask);
     void RecordResetEvent(CMD_TYPE cmd_type, VkEvent event, VkPipelineStageFlags2KHR stageMask);
@@ -567,25 +640,32 @@ class CMD_BUFFER_STATE : public REFCOUNTED_NODE {
         }
         return false;
     }
-
-    // For given pipeline, return number of MSAA samples, or one if MSAA disabled
-    VkSampleCountFlagBits GetRasterizationSamples(const PIPELINE_STATE &pipeline) const {
-        VkSampleCountFlagBits rasterization_samples = VK_SAMPLE_COUNT_1_BIT;
-        if (pipeline.IsDynamic(VK_DYNAMIC_STATE_RASTERIZATION_SAMPLES_EXT)) {
-            rasterization_samples = dynamic_state_value.rasterization_samples;
-        } else {
-            const auto ms_state = pipeline.MultisampleState();
-            if (ms_state) {
-                rasterization_samples = ms_state->rasterizationSamples;
+    bool HasDynamicDualSourceBlend(uint32_t attachmentCount) const {
+        if (dynamic_state_value.color_blend_enabled.any()) {
+            if (dynamic_state_status.cb[CB_DYNAMIC_STATE_COLOR_BLEND_EQUATION_EXT]) {
+                for (uint32_t i = 0; i < dynamic_state_value.color_blend_equations.size() && i < attachmentCount; ++i) {
+                    const auto &color_blend_equation = dynamic_state_value.color_blend_equations[i];
+                    if (IsSecondaryColorInputBlendFactor(color_blend_equation.srcColorBlendFactor) ||
+                        IsSecondaryColorInputBlendFactor(color_blend_equation.dstColorBlendFactor) ||
+                        IsSecondaryColorInputBlendFactor(color_blend_equation.srcAlphaBlendFactor) ||
+                        IsSecondaryColorInputBlendFactor(color_blend_equation.dstAlphaBlendFactor)) {
+                        return true;
+                    }
+                }
             }
         }
-        return rasterization_samples;
+        return false;
     }
 
-    bool RasterizationDisabled() const;
     inline void BindPipeline(LvlBindPoint bind_point, PIPELINE_STATE *pipe_state) {
         lastBound[bind_point].pipeline_state = pipe_state;
     }
+    void BindShader(VkShaderStageFlagBits shader_stage, SHADER_OBJECT_STATE *shader_object_state) {
+        auto &lastBoundState = lastBound[ConvertToPipelineBindPoint(shader_stage)];
+        const auto stage_index = static_cast<uint32_t>(ConvertToShaderObjectStage(shader_stage));
+        lastBoundState.shader_object_bound[stage_index] = true;
+        lastBoundState.shader_object_states[stage_index] = shader_object_state;
+    }
 
     bool IsPrimary() const { return createInfo.level == VK_COMMAND_BUFFER_LEVEL_PRIMARY; }
     void BeginLabel() { ++label_stack_depth_; }
