@@ -90,7 +90,7 @@ VkImageSubresourceRange NormalizeSubresourceRange(const VkImageCreateInfo &image
     return NormalizeSubresourceRange(image_create_info, subres_range);
 }
 
-static VkExternalMemoryHandleTypeFlags GetExternalHandleType(const VkImageCreateInfo *pCreateInfo) {
+static VkExternalMemoryHandleTypeFlags GetExternalHandleTypes(const VkImageCreateInfo *pCreateInfo) {
     const auto *external_memory_info = LvlFindInChain<VkExternalMemoryImageCreateInfo>(pCreateInfo->pNext);
     return external_memory_info ? external_memory_info->handleTypes : 0;
 }
@@ -149,10 +149,9 @@ static IMAGE_STATE::MemoryReqs GetMemoryRequirements(const ValidationStateTracke
     return result;
 }
 
-static IMAGE_STATE::SparseReqs GetSparseRequirements(const ValidationStateTracker *dev_data, VkImage img,
-                                                     const VkImageCreateInfo *create_info) {
+static IMAGE_STATE::SparseReqs GetSparseRequirements(const ValidationStateTracker *dev_data, VkImage img, bool sparse_residency) {
     IMAGE_STATE::SparseReqs result;
-    if (create_info->flags & VK_IMAGE_CREATE_SPARSE_RESIDENCY_BIT) {
+    if (sparse_residency) {
         uint32_t count = 0;
         DispatchGetImageSparseMemoryRequirements(dev_data->device, img, &count, nullptr);
         result.resize(count);
@@ -189,7 +188,7 @@ static bool GetMetalExport(const VkImageCreateInfo *info, VkExportMetalObjectTyp
 IMAGE_STATE::IMAGE_STATE(const ValidationStateTracker *dev_data, VkImage img, const VkImageCreateInfo *pCreateInfo,
                          VkFormatFeatureFlags2KHR ff)
     : BINDABLE(img, kVulkanObjectTypeImage, (pCreateInfo->flags & VK_IMAGE_CREATE_SPARSE_BINDING_BIT) != 0,
-               (pCreateInfo->flags & VK_IMAGE_CREATE_PROTECTED_BIT) == 0, GetExternalHandleType(pCreateInfo)),
+               (pCreateInfo->flags & VK_IMAGE_CREATE_PROTECTED_BIT) == 0, GetExternalHandleTypes(pCreateInfo)),
       safe_create_info(pCreateInfo),
       createInfo(*safe_create_info.ptr()),
       shared_presentable(false),
@@ -202,8 +201,8 @@ IMAGE_STATE::IMAGE_STATE(const ValidationStateTracker *dev_data, VkImage img, co
       format_features(ff),
       disjoint((pCreateInfo->flags & VK_IMAGE_CREATE_DISJOINT_BIT) != 0),
       requirements(GetMemoryRequirements(dev_data, img, pCreateInfo, disjoint, IsExternalAHB())),
-      memory_requirements_checked{{false, false, false}},
-      sparse_requirements(GetSparseRequirements(dev_data, img, pCreateInfo)),
+      sparse_residency((pCreateInfo->flags & VK_IMAGE_CREATE_SPARSE_RESIDENCY_BIT) != 0),
+      sparse_requirements(GetSparseRequirements(dev_data, img, sparse_residency)),
       sparse_metadata_required(SparseMetaDataRequired(sparse_requirements)),
       get_sparse_reqs_called(false),
       sparse_metadata_bound(false),
@@ -221,7 +220,7 @@ IMAGE_STATE::IMAGE_STATE(const ValidationStateTracker *dev_data, VkImage img, co
 IMAGE_STATE::IMAGE_STATE(const ValidationStateTracker *dev_data, VkImage img, const VkImageCreateInfo *pCreateInfo,
                          VkSwapchainKHR swapchain, uint32_t swapchain_index, VkFormatFeatureFlags2KHR ff)
     : BINDABLE(img, kVulkanObjectTypeImage, (pCreateInfo->flags & VK_IMAGE_CREATE_SPARSE_BINDING_BIT) != 0,
-               (pCreateInfo->flags & VK_IMAGE_CREATE_PROTECTED_BIT) == 0, GetExternalHandleType(pCreateInfo)),
+               (pCreateInfo->flags & VK_IMAGE_CREATE_PROTECTED_BIT) == 0, GetExternalHandleTypes(pCreateInfo)),
       safe_create_info(pCreateInfo),
       createInfo(*safe_create_info.ptr()),
       shared_presentable(false),
@@ -234,7 +233,7 @@ IMAGE_STATE::IMAGE_STATE(const ValidationStateTracker *dev_data, VkImage img, co
       format_features(ff),
       disjoint((pCreateInfo->flags & VK_IMAGE_CREATE_DISJOINT_BIT) != 0),
       requirements{},
-      memory_requirements_checked{false, false, false},
+      sparse_residency(false),
       sparse_requirements{},
       sparse_metadata_required(false),
       get_sparse_reqs_called(false),
@@ -303,7 +302,7 @@ bool IMAGE_STATE::IsCreateInfoDedicatedAllocationImageAliasingCompatible(const V
     return is_compatible;
 }
 
-bool IMAGE_STATE::IsCompatibleAliasing(IMAGE_STATE *other_image_state) const {
+bool IMAGE_STATE::IsCompatibleAliasing(const IMAGE_STATE *other_image_state) const {
     if (!IsSwapchainImage() && !other_image_state->IsSwapchainImage() &&
         !(createInfo.flags & other_image_state->createInfo.flags & VK_IMAGE_CREATE_ALIAS_BIT)) {
         return false;
@@ -326,53 +325,42 @@ void IMAGE_STATE::SetInitialLayoutMap() {
     if (layout_range_map) {
         return;
     }
-    if ((createInfo.flags & VK_IMAGE_CREATE_ALIAS_BIT) != 0) {
-        // Look for another aliasing image and point at its layout state.
-        // ObjectBindings() is thread safe since returns by value, and once
-        // the weak_ptr is successfully locked, the other image state won't
-        // be freed out from under us.
-        for (auto const &memory_state : GetBoundMemoryStates()) {
-            for (auto &entry : memory_state->ObjectBindings()) {
-                if (entry.first.type == kVulkanObjectTypeImage) {
-                    auto base_node = entry.second.lock();
-                    if (base_node) {
-                        auto other_image = static_cast<IMAGE_STATE *>(base_node.get());
-                        if (other_image != this && other_image->IsCompatibleAliasing(this)) {
-                            layout_range_map = other_image->layout_range_map;
-                            break;
-                        }
-                    }
-                }
-            }
-        }
+
+    std::shared_ptr<GlobalImageLayoutRangeMap> layout_map;
+    auto get_layout_map = [&layout_map](const IMAGE_STATE &other_image) {
+        layout_map = other_image.layout_range_map;
+        return true;
+    };
+
+    // See if an alias already has a layout map
+    if (HasAliasFlag()) {
+        AnyImageAliasOf(get_layout_map);
     } else if (bind_swapchain) {
         // Swapchains can also alias if multiple images are bound (or retrieved
         // with vkGetSwapchainImages()) for a (single swapchain, index) pair.
-        // ObjectBindings() is thread safe since returns by value, and once
-        // the weak_ptr is successfully locked, the other image state won't
-        // be freed out from under us.
-        for (auto &entry : bind_swapchain->ObjectBindings()) {
-            if (entry.first.type == kVulkanObjectTypeImage) {
-                auto base_node = entry.second.lock();
-                if (base_node) {
-                    auto other_image = static_cast<IMAGE_STATE *>(base_node.get());
-                    if (other_image != this && other_image->IsCompatibleAliasing(this)) {
-                        layout_range_map = other_image->layout_range_map;
-                        break;
-                    }
-                }
-            }
-        }
+        AnyAliasBindingOf(bind_swapchain->ObjectBindings(), get_layout_map);
     }
-    // ... otherwise set up the new map.
-    if (!layout_range_map) {
+
+    if (!layout_map) {
+        // otherwise set up a new map.
         // set up the new map completely before making it available
-        auto new_map = std::make_shared<GlobalImageLayoutRangeMap>(subresource_encoder.SubresourceCount());
+        layout_map = std::make_shared<GlobalImageLayoutRangeMap>(subresource_encoder.SubresourceCount());
         auto range_gen = subresource_adapter::RangeGenerator(subresource_encoder);
         for (; range_gen->non_empty(); ++range_gen) {
-            new_map->insert(new_map->end(), std::make_pair(*range_gen, createInfo.initialLayout));
+            layout_map->insert(layout_map->end(), std::make_pair(*range_gen, createInfo.initialLayout));
         }
-        layout_range_map = std::move(new_map);
+    }
+    // And store in the object
+    layout_range_map = std::move(layout_map);
+}
+
+void IMAGE_STATE::SetImageLayout(const VkImageSubresourceRange &range, VkImageLayout layout) {
+    using sparse_container::update_range_value;
+    using sparse_container::value_precedence;
+    GlobalImageLayoutRangeMap::RangeGenerator range_gen(subresource_encoder, NormalizeSubresourceRange(range));
+    auto guard = layout_range_map->WriteLock();
+    for (; range_gen->non_empty(); ++range_gen) {
+        update_range_value(*layout_range_map, *range_gen, layout, value_precedence::prefer_source);
     }
 }
 
@@ -383,16 +371,6 @@ void IMAGE_STATE::SetSwapchain(std::shared_ptr<SWAPCHAIN_NODE> &swapchain, uint3
     bind_swapchain->AddParent(this);
 }
 
-VkDeviceSize IMAGE_STATE::GetFakeBaseAddress() const {
-    if (!IsSwapchainImage()) {
-        return BINDABLE::GetFakeBaseAddress();
-    }
-    if (!bind_swapchain) {
-        return 0;
-    }
-    return bind_swapchain->images[swapchain_image_index].fake_base_address;
-}
-
 static VkSamplerYcbcrConversion GetSamplerConversion(const VkImageViewCreateInfo *ci) {
     auto *conversion_info = LvlFindInChain<VkSamplerYcbcrConversionInfo>(ci->pNext);
     return conversion_info ? conversion_info->conversion : VK_NULL_HANDLE;
@@ -434,8 +412,7 @@ IMAGE_VIEW_STATE::IMAGE_VIEW_STATE(const std::shared_ptr<IMAGE_STATE> &im, VkIma
       // When the image has a external format the views format must be VK_FORMAT_UNDEFINED and it is required to use a sampler
       // Ycbcr conversion. Thus we can't extract any meaningful information from the format parameter. As a Sampler Ycbcr
       // conversion must be used the shader type is always float.
-      descriptor_format_bits(im->HasAHBFormat() ? static_cast<unsigned>(DESCRIPTOR_REQ_COMPONENT_TYPE_FLOAT)
-                                                : DescriptorRequirementsBitsFromFormat(ci->format)),
+      descriptor_format_bits(im->HasAHBFormat() ? static_cast<unsigned>(NumericTypeFloat) : GetFormatType(ci->format)),
       samplerConversion(GetSamplerConversion(ci)),
       filter_cubic_props(cubic_props),
       min_lod(GetImageViewMinLod(ci)),
@@ -574,9 +551,10 @@ SWAPCHAIN_NODE::SWAPCHAIN_NODE(ValidationStateTracker *dev_data_, const VkSwapch
 void SWAPCHAIN_NODE::PresentImage(uint32_t image_index, uint64_t present_id) {
     if (image_index >= images.size()) return;
     assert(acquired_images > 0);
-    acquired_images--;
-    images[image_index].acquired = false;
-    if (shared_presentable) {
+    if (!shared_presentable) {
+        acquired_images--;
+        images[image_index].acquired = false;
+    } else {
         IMAGE_STATE *image_state = images[image_index].image_state;
         if (image_state) {
             image_state->layout_locked = true;
@@ -954,3 +932,15 @@ VkSurfacePresentScalingCapabilitiesEXT SURFACE_STATE::GetPresentModeScalingCapab
     DispatchGetPhysicalDeviceSurfaceCapabilities2KHR(phys_dev, &surface_info, &surface_capabilities);
     return scaling_caps;
 }
+
+bool GlobalImageLayoutRangeMap::AnyInRange(RangeGenerator &gen,
+                                           std::function<bool(const key_type &range, const mapped_type &state)> &&func) const {
+    for (; gen->non_empty(); ++gen) {
+        for (auto pos = lower_bound(*gen); (pos != end()) && (gen->intersects(pos->first)); ++pos) {
+            if (func(pos->first, pos->second)) {
+                return true;
+            }
+        }
+    }
+    return false;
+}
