@@ -20,21 +20,27 @@
 #include "state_tracker/descriptor_sets.h"
 #include "state_tracker/cmd_buffer_state.h"
 #include "generated/enum_flag_bits.h"
+#include "state_tracker/shader_object_state.h"
 
-PipelineStageState::PipelineStageState(const safe_VkPipelineShaderStageCreateInfo *create_info,
-                                       std::shared_ptr<const SHADER_MODULE_STATE> &module_state,
-                                       std::shared_ptr<const SHADER_MODULE_STATE::EntryPoint> &entrypoint)
-    : module_state(module_state), create_info(create_info), entrypoint(entrypoint) {
-    if (entrypoint) {
-        descriptor_variables = &(*entrypoint).resource_interface_variables;
+std::vector<VkPushConstantRange> const *StageCreateInfo::GetPushConstantRanges() const {
+    if (pipeline) {
+        return pipeline->PipelineLayoutState()->push_constant_ranges.get();
     }
+    return shader_object_const_ranges.get();
 }
 
+StageCreateInfo::StageCreateInfo(const std::string &name, const PIPELINE_STATE *pipeline)
+    : func_name(name), create_index(pipeline->create_index), pipeline(pipeline) {}
+StageCreateInfo::StageCreateInfo(const std::string &name, uint32_t create_index, const VkShaderCreateInfoEXT &create_info)
+    : func_name(name),
+      create_index(create_index),
+      pipeline(nullptr),
+      shader_object_const_ranges(GetCanonicalId(create_info.pushConstantRangeCount, create_info.pPushConstantRanges)) {}
+
 // static
-PIPELINE_STATE::StageStateVec PIPELINE_STATE::GetStageStates(const ValidationStateTracker &state_data,
-                                                             const PIPELINE_STATE &pipe_state,
-                                                             CreateShaderModuleStates *csm_states) {
-    PIPELINE_STATE::StageStateVec stage_states;
+StageStateVec PIPELINE_STATE::GetStageStates(const ValidationStateTracker &state_data, const PIPELINE_STATE &pipe_state,
+                                             CreateShaderModuleStates *csm_states) {
+    StageStateVec stage_states;
 
     // stages such as VK_SHADER_STAGE_ALL are find as this code is only looking for exact matches, not bool logic
     for (const auto &stage : AllVkShaderStageFlags) {
@@ -42,29 +48,28 @@ PIPELINE_STATE::StageStateVec PIPELINE_STATE::GetStageStates(const ValidationSta
         // shader stages need to be recorded in pipeline order
         for (const auto &stage_ci : pipe_state.shader_stages_ci) {
             if (stage_ci.stage == stage) {
-                auto module = state_data.Get<SHADER_MODULE_STATE>(stage_ci.module);
-                if (!module) {
+                auto module_state = state_data.Get<SHADER_MODULE_STATE>(stage_ci.module);
+                if (!module_state) {
                     // See if the module is referenced in a library sub state
-                    module = pipe_state.GetSubStateShader(stage_ci.stage);
+                    module_state = pipe_state.GetSubStateShader(stage_ci.stage);
                 }
 
-                if (!module) {
+                if (!module_state || !module_state->spirv) {
                     // If module is null and there is a VkShaderModuleCreateInfo in the pNext chain of the stage info, then this
                     // module is part of a library and the state must be created
                     const auto shader_ci = LvlFindInChain<VkShaderModuleCreateInfo>(stage_ci.pNext);
                     const uint32_t unique_shader_id = (csm_states) ? (*csm_states)[stage].unique_shader_id : 0;
                     if (shader_ci) {
-                        module = state_data.CreateShaderModuleState(*shader_ci, unique_shader_id);
+                        // don't need to worry about GroupDecoration in GPL
+                        auto spirv_module = std::make_shared<SPIRV_MODULE_STATE>(shader_ci->codeSize, shader_ci->pCode);
+                        module_state = std::make_shared<SHADER_MODULE_STATE>(VK_NULL_HANDLE, spirv_module, unique_shader_id);
                     } else {
-                        // shader_module_identifier could legally provide a null module handle
-                        VkShaderModuleCreateInfo dummy_module_ci = LvlInitStruct<VkShaderModuleCreateInfo>();
-                        dummy_module_ci.pCode = &unique_shader_id;  // Ensure tripping invalid spirv
-                        module = state_data.CreateShaderModuleState(dummy_module_ci, unique_shader_id);
+                        // VK_EXT_shader_module_identifier could legally provide a null module handle
+                        module_state = std::make_shared<SHADER_MODULE_STATE>(unique_shader_id);
                     }
                 }
 
-                auto entrypoint = module->FindEntrypoint(stage_ci.pName, stage_ci.stage);
-                stage_states.emplace_back(&stage_ci, module, entrypoint);
+                stage_states.emplace_back(&stage_ci, nullptr, module_state, module_state->spirv);
                 stage_found = true;
             }
         }
@@ -122,63 +127,13 @@ PIPELINE_STATE::StageStateVec PIPELINE_STATE::GetStageStates(const ValidationSta
             if (!stage_ci) {
                 continue;
             }
-            auto entrypoint = module_state->FindEntrypoint(stage_ci->pName, stage_ci->stage);
-            stage_states.emplace_back(stage_ci, module_state, entrypoint);
+
+            stage_states.emplace_back(stage_ci, nullptr, module_state, module_state->spirv);
         }
     }
     return stage_states;
 }
 
-// static
-PIPELINE_STATE::ActiveSlotMap PIPELINE_STATE::GetActiveSlots(const StageStateVec &stage_states) {
-    PIPELINE_STATE::ActiveSlotMap active_slots;
-    for (const auto &stage : stage_states) {
-        if (!stage.entrypoint || !stage.descriptor_variables) {
-            continue;
-        }
-        // Capture descriptor uses for the pipeline
-        for (const auto &variable : *stage.descriptor_variables) {
-            // While validating shaders capture which slots are used by the pipeline
-            auto &entry = active_slots[variable.decorations.set][variable.decorations.binding];
-            entry.is_written_to |= variable.is_written_to;
-
-            auto &reqs = entry.reqs;
-            reqs |= stage.module_state->DescriptorTypeToReqs(variable.type_id);
-            if (variable.is_atomic_operation) reqs |= DESCRIPTOR_REQ_VIEW_ATOMIC_OPERATION;
-            if (variable.is_sampler_sampled) reqs |= DESCRIPTOR_REQ_SAMPLER_SAMPLED;
-            if (variable.is_sampler_implicitLod_dref_proj) reqs |= DESCRIPTOR_REQ_SAMPLER_IMPLICITLOD_DREF_PROJ;
-            if (variable.is_sampler_bias_offset) reqs |= DESCRIPTOR_REQ_SAMPLER_BIAS_OFFSET;
-            if (variable.is_read_without_format) reqs |= DESCRIPTOR_REQ_IMAGE_READ_WITHOUT_FORMAT;
-            if (variable.is_write_without_format) reqs |= DESCRIPTOR_REQ_IMAGE_WRITE_WITHOUT_FORMAT;
-            if (variable.is_dref_operation) reqs |= DESCRIPTOR_REQ_IMAGE_DREF;
-
-            if (!variable.samplers_used_by_image.empty()) {
-                if (variable.samplers_used_by_image.size() > entry.samplers_used_by_image.size()) {
-                    entry.samplers_used_by_image.resize(variable.samplers_used_by_image.size());
-                }
-                uint32_t image_index = 0;
-                for (const auto &samplers : variable.samplers_used_by_image) {
-                    for (const auto &sampler : samplers) {
-                        entry.samplers_used_by_image[image_index].emplace(sampler);
-                    }
-                    ++image_index;
-                }
-            }
-            entry.write_without_formats_component_count_list = variable.write_without_formats_component_count_list;
-            entry.image_sampled_type_width = variable.image_sampled_type_width;
-        }
-    }
-    return active_slots;
-}
-
-static uint32_t GetMaxActiveSlot(const PIPELINE_STATE::ActiveSlotMap &active_slots) {
-    uint32_t max_active_slot = 0;
-    for (const auto &entry : active_slots) {
-        max_active_slot = std::max(max_active_slot, entry.first);
-    }
-    return max_active_slot;
-}
-
 static uint32_t GetCreateInfoShaders(const PIPELINE_STATE &pipe_state) {
     uint32_t result = 0;
     for (const auto &stage_ci : pipe_state.shader_stages_ci) {
@@ -200,6 +155,174 @@ static uint32_t GetLinkingShaders(const VkPipelineLibraryCreateInfoKHR *link_inf
     return result;
 }
 
+static CBDynamicFlags GetGraphicsDynamicState(PIPELINE_STATE &pipe_state) {
+    CBDynamicFlags flags = 0;
+
+    // "Dynamic state values set via pDynamicState must be ignored if the state they correspond to is not otherwise statically set
+    // by one of the state subsets used to create the pipeline."
+    //
+    // we only care here if the pipeline was created with the subset, not linked
+    const bool has_vertex_input_state = pipe_state.OwnsSubState(pipe_state.vertex_input_state);
+    const bool has_pre_raster_state = pipe_state.OwnsSubState(pipe_state.pre_raster_state);
+    const bool has_fragment_shader_state = pipe_state.OwnsSubState(pipe_state.fragment_shader_state);
+    const bool has_fragment_output_state = pipe_state.OwnsSubState(pipe_state.fragment_output_state);
+
+    const auto *dynamic_state_ci = pipe_state.DynamicState();
+    if (dynamic_state_ci) {
+        for (uint32_t i = 0; i < dynamic_state_ci->dynamicStateCount; i++) {
+            const VkDynamicState vk_dynamic_state = dynamic_state_ci->pDynamicStates[i];
+            // Check if should ignore or not before converting and adding
+            switch (vk_dynamic_state) {
+                // VkPipelineVertexInputStateCreateInfo
+                case VK_DYNAMIC_STATE_VERTEX_INPUT_EXT:
+                case VK_DYNAMIC_STATE_VERTEX_INPUT_BINDING_STRIDE:
+                // VkPipelineInputAssemblyStateCreateInfo
+                case VK_DYNAMIC_STATE_PRIMITIVE_TOPOLOGY:
+                case VK_DYNAMIC_STATE_PRIMITIVE_RESTART_ENABLE: {
+                    if (has_vertex_input_state) {
+                        flags.set(ConvertToCBDynamicState(vk_dynamic_state));
+                    }
+                    break;
+                }
+
+                // VkPipelineViewportStateCreateInfo
+                case VK_DYNAMIC_STATE_VIEWPORT:
+                case VK_DYNAMIC_STATE_SCISSOR:
+                case VK_DYNAMIC_STATE_VIEWPORT_WITH_COUNT:
+                case VK_DYNAMIC_STATE_SCISSOR_WITH_COUNT:
+                case VK_DYNAMIC_STATE_VIEWPORT_W_SCALING_NV:
+                case VK_DYNAMIC_STATE_VIEWPORT_SHADING_RATE_PALETTE_NV:
+                case VK_DYNAMIC_STATE_SHADING_RATE_IMAGE_ENABLE_NV:
+                case VK_DYNAMIC_STATE_VIEWPORT_COARSE_SAMPLE_ORDER_NV:
+                case VK_DYNAMIC_STATE_VIEWPORT_W_SCALING_ENABLE_NV:
+                case VK_DYNAMIC_STATE_VIEWPORT_SWIZZLE_NV:
+                case VK_DYNAMIC_STATE_DEPTH_CLIP_NEGATIVE_ONE_TO_ONE_EXT:
+                case VK_DYNAMIC_STATE_EXCLUSIVE_SCISSOR_ENABLE_NV:
+                case VK_DYNAMIC_STATE_EXCLUSIVE_SCISSOR_NV:
+                // VkPipelineRasterizationStateCreateInfo
+                case VK_DYNAMIC_STATE_LINE_WIDTH:
+                case VK_DYNAMIC_STATE_DEPTH_BIAS:
+                case VK_DYNAMIC_STATE_CULL_MODE:
+                case VK_DYNAMIC_STATE_FRONT_FACE:
+                case VK_DYNAMIC_STATE_RASTERIZER_DISCARD_ENABLE:
+                case VK_DYNAMIC_STATE_DEPTH_BIAS_ENABLE:
+                case VK_DYNAMIC_STATE_DEPTH_CLAMP_ENABLE_EXT:
+                case VK_DYNAMIC_STATE_POLYGON_MODE_EXT:
+                case VK_DYNAMIC_STATE_LINE_STIPPLE_EXT:
+                case VK_DYNAMIC_STATE_LINE_RASTERIZATION_MODE_EXT:
+                case VK_DYNAMIC_STATE_LINE_STIPPLE_ENABLE_EXT:
+                case VK_DYNAMIC_STATE_RASTERIZATION_STREAM_EXT:
+                case VK_DYNAMIC_STATE_CONSERVATIVE_RASTERIZATION_MODE_EXT:
+                case VK_DYNAMIC_STATE_EXTRA_PRIMITIVE_OVERESTIMATION_SIZE_EXT:
+                case VK_DYNAMIC_STATE_DEPTH_CLIP_ENABLE_EXT:
+                case VK_DYNAMIC_STATE_PROVOKING_VERTEX_MODE_EXT:
+                // VkPipelineTessellationStateCreateInfo
+                case VK_DYNAMIC_STATE_PATCH_CONTROL_POINTS_EXT:
+                case VK_DYNAMIC_STATE_TESSELLATION_DOMAIN_ORIGIN_EXT:
+                // VkPipelineDiscardRectangleStateCreateInfoEXT
+                case VK_DYNAMIC_STATE_DISCARD_RECTANGLE_EXT:
+                case VK_DYNAMIC_STATE_DISCARD_RECTANGLE_ENABLE_EXT:
+                case VK_DYNAMIC_STATE_DISCARD_RECTANGLE_MODE_EXT: {
+                    if (has_pre_raster_state) {
+                        flags.set(ConvertToCBDynamicState(vk_dynamic_state));
+                    }
+                    break;
+                }
+
+                // VkPipelineFragmentShadingRateStateCreateInfoKHR
+                case VK_DYNAMIC_STATE_FRAGMENT_SHADING_RATE_KHR: {
+                    if (has_pre_raster_state || has_fragment_shader_state) {
+                        flags.set(ConvertToCBDynamicState(vk_dynamic_state));
+                    }
+                    break;
+                }
+
+                // VkPipelineDepthStencilStateCreateInfo
+                case VK_DYNAMIC_STATE_DEPTH_BOUNDS:
+                case VK_DYNAMIC_STATE_DEPTH_BOUNDS_TEST_ENABLE:
+                case VK_DYNAMIC_STATE_STENCIL_COMPARE_MASK:
+                case VK_DYNAMIC_STATE_STENCIL_WRITE_MASK:
+                case VK_DYNAMIC_STATE_STENCIL_REFERENCE:
+                case VK_DYNAMIC_STATE_DEPTH_TEST_ENABLE:
+                case VK_DYNAMIC_STATE_DEPTH_WRITE_ENABLE:
+                case VK_DYNAMIC_STATE_DEPTH_COMPARE_OP:
+                case VK_DYNAMIC_STATE_STENCIL_TEST_ENABLE:
+                case VK_DYNAMIC_STATE_STENCIL_OP:
+                // VkPipelineRepresentativeFragmentTestStateCreateInfoNV
+                case VK_DYNAMIC_STATE_REPRESENTATIVE_FRAGMENT_TEST_ENABLE_NV: {
+                    if (has_fragment_shader_state) {
+                        flags.set(ConvertToCBDynamicState(vk_dynamic_state));
+                    }
+                    break;
+                }
+
+                /// VkPipelineColorBlendStateCreateInfo
+                case VK_DYNAMIC_STATE_BLEND_CONSTANTS:
+                case VK_DYNAMIC_STATE_LOGIC_OP_EXT:
+                case VK_DYNAMIC_STATE_LOGIC_OP_ENABLE_EXT:
+                case VK_DYNAMIC_STATE_COLOR_WRITE_ENABLE_EXT:
+                case VK_DYNAMIC_STATE_COLOR_BLEND_ENABLE_EXT:
+                case VK_DYNAMIC_STATE_COLOR_BLEND_EQUATION_EXT:
+                case VK_DYNAMIC_STATE_COLOR_BLEND_ADVANCED_EXT:
+                case VK_DYNAMIC_STATE_COLOR_WRITE_MASK_EXT: {
+                    if (has_fragment_output_state) {
+                        flags.set(ConvertToCBDynamicState(vk_dynamic_state));
+                    }
+                    break;
+                }
+
+                // VkPipelineMultisampleStateCreateInfo
+                case VK_DYNAMIC_STATE_RASTERIZATION_SAMPLES_EXT:
+                case VK_DYNAMIC_STATE_SAMPLE_MASK_EXT:
+                case VK_DYNAMIC_STATE_ALPHA_TO_COVERAGE_ENABLE_EXT:
+                case VK_DYNAMIC_STATE_ALPHA_TO_ONE_ENABLE_EXT:
+                case VK_DYNAMIC_STATE_SAMPLE_LOCATIONS_EXT:
+                case VK_DYNAMIC_STATE_SAMPLE_LOCATIONS_ENABLE_EXT:
+                case VK_DYNAMIC_STATE_COVERAGE_TO_COLOR_ENABLE_NV:
+                case VK_DYNAMIC_STATE_COVERAGE_TO_COLOR_LOCATION_NV:
+                case VK_DYNAMIC_STATE_COVERAGE_MODULATION_MODE_NV:
+                case VK_DYNAMIC_STATE_COVERAGE_MODULATION_TABLE_ENABLE_NV:
+                case VK_DYNAMIC_STATE_COVERAGE_MODULATION_TABLE_NV:
+                case VK_DYNAMIC_STATE_COVERAGE_REDUCTION_MODE_NV: {
+                    if (has_fragment_shader_state || has_fragment_output_state) {
+                        flags.set(ConvertToCBDynamicState(vk_dynamic_state));
+                    }
+                    break;
+                }
+
+                // VkRenderPass and subpass parameter
+                case VK_DYNAMIC_STATE_ATTACHMENT_FEEDBACK_LOOP_ENABLE_EXT: {
+                    if (has_pre_raster_state || has_fragment_shader_state || has_fragment_output_state) {
+                        flags.set(ConvertToCBDynamicState(vk_dynamic_state));
+                    }
+                    break;
+                }
+
+                // not valid and shouldn't see
+                case VK_DYNAMIC_STATE_RAY_TRACING_PIPELINE_STACK_SIZE_KHR:
+                case VK_DYNAMIC_STATE_MAX_ENUM:
+                    assert(false);
+                    break;
+            }
+        }
+    }
+
+    // apply linked library's dynamic state
+    if (!has_vertex_input_state && pipe_state.vertex_input_state) {
+        flags |= pipe_state.vertex_input_state->parent.dynamic_state;
+    }
+    if (!has_pre_raster_state && pipe_state.pre_raster_state) {
+        flags |= pipe_state.pre_raster_state->parent.dynamic_state;
+    }
+    if (!has_fragment_shader_state && pipe_state.fragment_shader_state) {
+        flags |= pipe_state.fragment_shader_state->parent.dynamic_state;
+    }
+    if (!has_fragment_output_state && pipe_state.fragment_output_state) {
+        flags |= pipe_state.fragment_output_state->parent.dynamic_state;
+    }
+    return flags;
+}
+
 static bool UsesPipelineRobustness(const void *pNext, const PIPELINE_STATE &pipe_state) {
     bool result = false;
     const auto robustness_info = LvlFindInChain<VkPipelineRobustnessCreateInfoEXT>(pNext);
@@ -226,6 +349,27 @@ static bool UsesPipelineRobustness(const void *pNext, const PIPELINE_STATE &pipe
     return result;
 }
 
+static bool IgnoreColorAttachments(const ValidationStateTracker *state_data, PIPELINE_STATE &pipe_state) {
+    bool ignore = false;
+    // If the libraries used to create this pipeline are ignoring color attachments, this pipeline should as well
+    if (pipe_state.library_create_info) {
+        for (uint32_t i = 0; i < pipe_state.library_create_info->libraryCount; i++) {
+            const auto lib = state_data->Get<PIPELINE_STATE>(pipe_state.library_create_info->pLibraries[i]);
+            if (lib->ignore_color_attachments) return true;
+        }
+    }
+    // According to the spec, pAttachments is to be ignored if the pipeline is created with
+    // VK_DYNAMIC_STATE_COLOR_BLEND_ENABLE_EXT, VK_DYNAMIC_STATE_COLOR_BLEND_EQUATION_EXT, VK_DYNAMIC_STATE_COLOR_BLEND_ADVANCED_EXT
+    // and VK_DYNAMIC_STATE_COLOR_WRITE_MASK_EXT dynamic states set
+    if (pipe_state.ColorBlendState() && pipe_state.DynamicState()) {
+        ignore = (pipe_state.IsDynamic(VK_DYNAMIC_STATE_COLOR_BLEND_ENABLE_EXT) &&
+                  pipe_state.IsDynamic(VK_DYNAMIC_STATE_COLOR_BLEND_EQUATION_EXT) &&
+                  pipe_state.IsDynamic(VK_DYNAMIC_STATE_COLOR_BLEND_ADVANCED_EXT) &&
+                  pipe_state.IsDynamic(VK_DYNAMIC_STATE_COLOR_WRITE_MASK_EXT));
+    }
+    return ignore;
+}
+
 static bool UsesShaderModuleId(const PIPELINE_STATE &pipe_state) {
     for (const auto &stage_ci : pipe_state.shader_stages_ci) {
         const auto module_id_info = LvlFindInChain<VkPipelineShaderStageModuleIdentifierCreateInfoEXT>(stage_ci.pNext);
@@ -236,13 +380,13 @@ static bool UsesShaderModuleId(const PIPELINE_STATE &pipe_state) {
     return false;
 }
 
-static vvl::unordered_set<uint32_t> GetFSOutputLocations(const PIPELINE_STATE::StageStateVec &stage_states) {
+static vvl::unordered_set<uint32_t> GetFSOutputLocations(const StageStateVec &stage_states) {
     vvl::unordered_set<uint32_t> result;
     for (const auto &stage_state : stage_states) {
         if (!stage_state.entrypoint) {
             continue;
         }
-        if (stage_state.create_info->stage == VK_SHADER_STAGE_FRAGMENT_BIT) {
+        if (stage_state.getStage() == VK_SHADER_STAGE_FRAGMENT_BIT) {
             for (const auto *variable : stage_state.entrypoint->user_defined_interface_variables) {
                 if ((variable->storage_class != spv::StorageClassOutput) || variable->interface_slots.empty()) {
                     continue;  // not an output interface
@@ -265,7 +409,7 @@ static VkPrimitiveTopology GetTopologyAtRasterizer(const PIPELINE_STATE &pipelin
         if (!stage.entrypoint) {
             continue;
         }
-        auto stage_topo = stage.module_state->GetTopology(*stage.entrypoint);
+        auto stage_topo = stage.spirv_state->GetTopology(*stage.entrypoint);
         if (stage_topo) {
             result = *stage_topo;
         }
@@ -333,11 +477,12 @@ std::shared_ptr<FragmentShaderState> PIPELINE_STATE::CreateFragmentShaderState(
 
     if (p.library_create_info) {
         auto ss = GetLibSubState<VK_GRAPHICS_PIPELINE_LIBRARY_FRAGMENT_SHADER_BIT_EXT>(state, *p.library_create_info);
-        if (ss) {
+        if (ss && EnablesRasterizationStates(p.pre_raster_state)) {
             return ss;
         }
     } else {
-        if (lib_type == static_cast<VkGraphicsPipelineLibraryFlagsEXT>(0)) {  // Not a graphics library
+        if ((lib_type == static_cast<VkGraphicsPipelineLibraryFlagsEXT>(0)) &&  // Not a graphics library
+            EnablesRasterizationStates(p.pre_raster_state)) {
             // No fragment shader _should_ imply no fragment shader state, however, for historical (GL) reasons, a pipeline _can_
             // be created with a VS but no FS and still have valid fragment shader state.
             // See https://gitlab.khronos.org/vulkan/vulkan/-/issues/3178 for more details.
@@ -345,7 +490,7 @@ std::shared_ptr<FragmentShaderState> PIPELINE_STATE::CreateFragmentShaderState(
         }
     }
 
-    // We shouldn't get here...
+    // The conditions for containing FS state were not met, so return null
     return {};
 }
 
@@ -355,11 +500,8 @@ std::shared_ptr<FragmentShaderState> PIPELINE_STATE::CreateFragmentShaderState(
 std::shared_ptr<FragmentOutputState> PIPELINE_STATE::CreateFragmentOutputState(
     const PIPELINE_STATE &p, const ValidationStateTracker &state, const VkGraphicsPipelineCreateInfo &create_info,
     const safe_VkGraphicsPipelineCreateInfo &safe_create_info, const std::shared_ptr<const RENDER_PASS_STATE> &rp) {
-    // If we have rasterization state and rasterizerDiscardEnable is VK_TRUE, then there is no fragment output state
-    if (create_info.pRasterizationState && create_info.pRasterizationState->rasterizerDiscardEnable == VK_TRUE) {
-        return {};
-    }
-
+    // If this pipeline is being created a non-executable (i.e., does not contain complete state) pipeline with FO state, then
+    // unconditionally set this pipeline's FO state.
     const auto lib_type = GetGraphicsLibType(create_info);
     if (lib_type & VK_GRAPHICS_PIPELINE_LIBRARY_FRAGMENT_OUTPUT_INTERFACE_BIT_EXT) {  // Fragment output graphics library
         return std::make_shared<FragmentOutputState>(p, create_info, rp);
@@ -367,36 +509,24 @@ std::shared_ptr<FragmentOutputState> PIPELINE_STATE::CreateFragmentOutputState(
 
     if (p.library_create_info) {
         auto ss = GetLibSubState<VK_GRAPHICS_PIPELINE_LIBRARY_FRAGMENT_OUTPUT_INTERFACE_BIT_EXT>(state, *p.library_create_info);
-        if (ss) {
+        // If this pipeline is linking in a library that contains FO state, check to see if the FO state is valid before creating it
+        // for this pipeline
+        if (ss && EnablesRasterizationStates(p.pre_raster_state)) {
             return ss;
         }
     } else {
-        if (lib_type == static_cast<VkGraphicsPipelineLibraryFlagsEXT>(0)) {  // Not a graphics library
+        // This is a complete pipeline that does not link to any graphics libraries. Check its create info to see if it has valid FO
+        // state
+        if ((lib_type == static_cast<VkGraphicsPipelineLibraryFlagsEXT>(0)) &&  // Not a graphics library
+            EnablesRasterizationStates(p.pre_raster_state)) {
             return std::make_shared<FragmentOutputState>(p, safe_create_info, rp);
         }
     }
 
-    // We shouldn't get here...
+    // The conditions for containing FO state were not met, so return null
     return {};
 }
 
-template <typename Substate>
-void AppendDynamicStateFromSubstate(const Substate &substate, std::vector<VkDynamicState> &dyn_states,
-                                    VkPipelineDynamicStateCreateFlags &flags) {
-    if (substate) {
-        const auto *dyn_state = substate->parent.DynamicState();
-        if (dyn_state) {
-            flags |= dyn_state->flags;
-            for (uint32_t i = 0; i < dyn_state->dynamicStateCount; ++i) {
-                const auto itr = std::find(dyn_states.cbegin(), dyn_states.cend(), dyn_state->pDynamicStates[i]);
-                if (itr == dyn_states.cend()) {
-                    dyn_states.emplace_back(dyn_state->pDynamicStates[i]);
-                }
-            }
-        }
-    }
-}
-
 std::vector<std::shared_ptr<const PIPELINE_LAYOUT_STATE>> PIPELINE_STATE::PipelineLayoutStateUnion() const {
     std::vector<std::shared_ptr<const PIPELINE_LAYOUT_STATE>> ret;
     ret.reserve(2);
@@ -476,6 +606,7 @@ VkShaderModule PIPELINE_STATE::GetShaderModuleByCIIndex<VkRayTracingPipelineCrea
 }
 
 // TODO (ncesario) this needs to be automated. As a first step, need to leverage SubState::ValidShaderStages()
+// Currently will return SHADER_MODULE_STATE with no SPIR-V
 std::shared_ptr<const SHADER_MODULE_STATE> PIPELINE_STATE::GetSubStateShader(VkShaderStageFlagBits state) const {
     switch (state) {
         case VK_SHADER_STAGE_VERTEX_BIT: {
@@ -529,7 +660,7 @@ PIPELINE_STATE::PIPELINE_STATE(const ValidationStateTracker *state_data, const V
                                std::shared_ptr<const PIPELINE_LAYOUT_STATE> &&layout, CreateShaderModuleStates *csm_states)
     : BASE_NODE(static_cast<VkPipeline>(VK_NULL_HANDLE), kVulkanObjectTypePipeline),
       rp_state(rpstate),
-      create_info(pCreateInfo, rpstate),
+      create_info(*pCreateInfo, rpstate, state_data),
       create_index(create_index),
       rendering_create_info(LvlFindInChain<VkPipelineRenderingCreateInfo>(PNext())),
       library_create_info(LvlFindInChain<VkPipelineLibraryCreateInfoKHR>(PNext())),
@@ -549,44 +680,13 @@ PIPELINE_STATE::PIPELINE_STATE(const ValidationStateTracker *state_data, const V
       fragmentShader_writable_output_location_list(GetFSOutputLocations(stage_states)),
       active_slots(GetActiveSlots(stage_states)),
       max_active_slot(GetMaxActiveSlot(active_slots)),
+      dynamic_state(GetGraphicsDynamicState(*this)),
       topology_at_rasterizer(GetTopologyAtRasterizer(*this)),
       descriptor_buffer_mode((create_info.graphics.flags & VK_PIPELINE_CREATE_DESCRIPTOR_BUFFER_BIT_EXT) != 0),
       uses_pipeline_robustness(UsesPipelineRobustness(PNext(), *this)),
+      ignore_color_attachments(IgnoreColorAttachments(state_data, *this)),
       csm_states(csm_states) {
     if (library_create_info) {
-        // accumulate dynamic state
-        // TODO is this correct?
-        auto *dyn_state_ci = const_cast<safe_VkPipelineDynamicStateCreateInfo *>(create_info.graphics.pDynamicState);
-        std::vector<VkDynamicState> dyn_states;
-        VkPipelineDynamicStateCreateFlags dyn_flags = 0;
-        if (create_info.graphics.pDynamicState) {
-            std::copy(dyn_state_ci->pDynamicStates, dyn_state_ci->pDynamicStates + dyn_state_ci->dynamicStateCount,
-                      std::back_inserter(dyn_states));
-            dyn_flags = dyn_state_ci->flags;
-        }
-        AppendDynamicStateFromSubstate(vertex_input_state, dyn_states, dyn_flags);
-        AppendDynamicStateFromSubstate(pre_raster_state, dyn_states, dyn_flags);
-        AppendDynamicStateFromSubstate(fragment_shader_state, dyn_states, dyn_flags);
-        AppendDynamicStateFromSubstate(fragment_output_state, dyn_states, dyn_flags);
-        if (dyn_states.size() > 0) {
-            // We have dynamic state
-            if (!dyn_state_ci || (dyn_state_ci->dynamicStateCount < dyn_states.size())) {
-                // There is dynamic state defined in libraries that the is not included in this pipeline's create info
-                if (!dyn_state_ci) {
-                    // *All* dynamic state defined is coming from graphics libraries
-                    // NOTE: heap allocation cleaned up in ~safe_VkGraphicsPipelineCreateInfo
-                    dyn_state_ci = new safe_VkPipelineDynamicStateCreateInfo;
-                    const_cast<safe_VkGraphicsPipelineCreateInfo *>(&create_info.graphics)->pDynamicState = dyn_state_ci;
-                }
-                dyn_state_ci->flags = dyn_flags;
-                dyn_state_ci->dynamicStateCount = static_cast<uint32_t>(dyn_states.size());
-                // NOTE: heap allocation cleaned up in ~safe_VkPipelineDynamicStateCreateInfo
-                dyn_state_ci->pDynamicStates = new VkDynamicState[dyn_states.size()];
-                std::copy(&dyn_states.front(), &dyn_states.front() + dyn_states.size(),
-                          const_cast<VkDynamicState *>(dyn_state_ci->pDynamicStates));
-            }
-        }
-
         const auto &exe_layout_state = state_data->Get<PIPELINE_LAYOUT_STATE>(create_info.graphics.layout);
         const auto *exe_layout = exe_layout_state.get();
         const auto *pre_raster_layout =
@@ -626,8 +726,10 @@ PIPELINE_STATE::PIPELINE_STATE(const ValidationStateTracker *state_data, const V
       active_shaders(create_info_shaders),  // compute has no linking shaders
       active_slots(GetActiveSlots(stage_states)),
       max_active_slot(GetMaxActiveSlot(active_slots)),
+      dynamic_state(0),  // compute has no dynamic state
       descriptor_buffer_mode((create_info.compute.flags & VK_PIPELINE_CREATE_DESCRIPTOR_BUFFER_BIT_EXT) != 0),
       uses_pipeline_robustness(UsesPipelineRobustness(PNext(), *this)),
+      ignore_color_attachments(IgnoreColorAttachments(state_data, *this)),
       csm_states(csm_states),
       merged_graphics_layout(layout) {
     assert(active_shaders == VK_SHADER_STAGE_COMPUTE_BIT);
@@ -649,8 +751,10 @@ PIPELINE_STATE::PIPELINE_STATE(const ValidationStateTracker *state_data, const V
       active_shaders(create_info_shaders),  // RTX has no linking shaders
       active_slots(GetActiveSlots(stage_states)),
       max_active_slot(GetMaxActiveSlot(active_slots)),
+      dynamic_state(0),  // RTX has no dynamic states being validated
       descriptor_buffer_mode((create_info.raytracing.flags & VK_PIPELINE_CREATE_DESCRIPTOR_BUFFER_BIT_EXT) != 0),
       uses_pipeline_robustness(UsesPipelineRobustness(PNext(), *this)),
+      ignore_color_attachments(IgnoreColorAttachments(state_data, *this)),
       csm_states(csm_states),
       merged_graphics_layout(std::move(layout)) {
     assert(0 == (active_shaders &
@@ -674,8 +778,10 @@ PIPELINE_STATE::PIPELINE_STATE(const ValidationStateTracker *state_data, const V
       active_shaders(create_info_shaders),  // RTX has no linking shaders
       active_slots(GetActiveSlots(stage_states)),
       max_active_slot(GetMaxActiveSlot(active_slots)),
+      dynamic_state(0),  // RTX has no dynamic states being validated
       descriptor_buffer_mode((create_info.graphics.flags & VK_PIPELINE_CREATE_DESCRIPTOR_BUFFER_BIT_EXT) != 0),
       uses_pipeline_robustness(UsesPipelineRobustness(PNext(), *this)),
+      ignore_color_attachments(IgnoreColorAttachments(state_data, *this)),
       csm_states(csm_states),
       merged_graphics_layout(std::move(layout)) {
     assert(0 == (active_shaders &
@@ -706,3 +812,126 @@ void LAST_BOUND_STATE::Reset() {
     push_descriptor_set.reset();
     per_set.clear();
 }
+
+bool LAST_BOUND_STATE::IsDepthTestEnable() const {
+    return pipeline_state->IsDynamic(VK_DYNAMIC_STATE_DEPTH_TEST_ENABLE) ? cb_state.dynamic_state_value.depth_test_enable
+                                                                         : pipeline_state->DepthStencilState()->depthTestEnable;
+}
+
+bool LAST_BOUND_STATE::IsDepthWriteEnable() const {
+    // "Depth writes are always disabled when depthTestEnable is VK_FALSE"
+    if (!IsDepthTestEnable()) {
+        return false;
+    }
+    return pipeline_state->IsDynamic(VK_DYNAMIC_STATE_DEPTH_WRITE_ENABLE) ? cb_state.dynamic_state_value.depth_write_enable
+                                                                          : pipeline_state->DepthStencilState()->depthWriteEnable;
+}
+
+bool LAST_BOUND_STATE::IsStencilTestEnable() const {
+    return pipeline_state->IsDynamic(VK_DYNAMIC_STATE_STENCIL_TEST_ENABLE) ? cb_state.dynamic_state_value.stencil_test_enable
+                                                                           : pipeline_state->DepthStencilState()->stencilTestEnable;
+}
+
+VkStencilOpState LAST_BOUND_STATE::GetStencilOpStateFront() const {
+    VkStencilOpState front = pipeline_state->DepthStencilState()->front;
+    if (pipeline_state->IsDynamic(VK_DYNAMIC_STATE_STENCIL_WRITE_MASK)) {
+        front.writeMask = cb_state.dynamic_state_value.write_mask_front;
+    }
+    if (pipeline_state->IsDynamic(VK_DYNAMIC_STATE_STENCIL_OP)) {
+        front.failOp = cb_state.dynamic_state_value.fail_op_front;
+        front.passOp = cb_state.dynamic_state_value.pass_op_front;
+        front.depthFailOp = cb_state.dynamic_state_value.depth_fail_op_front;
+    }
+    return front;
+}
+
+VkStencilOpState LAST_BOUND_STATE::GetStencilOpStateBack() const {
+    VkStencilOpState back = pipeline_state->DepthStencilState()->back;
+    if (pipeline_state->IsDynamic(VK_DYNAMIC_STATE_STENCIL_WRITE_MASK)) {
+        back.writeMask = cb_state.dynamic_state_value.write_mask_back;
+    }
+    if (pipeline_state->IsDynamic(VK_DYNAMIC_STATE_STENCIL_OP)) {
+        back.failOp = cb_state.dynamic_state_value.fail_op_back;
+        back.passOp = cb_state.dynamic_state_value.pass_op_back;
+        back.depthFailOp = cb_state.dynamic_state_value.depth_fail_op_back;
+    }
+    return back;
+}
+
+VkSampleCountFlagBits LAST_BOUND_STATE::GetRasterizationSamples() const {
+    // For given pipeline, return number of MSAA samples, or one if MSAA disabled
+    VkSampleCountFlagBits rasterization_samples = VK_SAMPLE_COUNT_1_BIT;
+    if (pipeline_state->IsDynamic(VK_DYNAMIC_STATE_RASTERIZATION_SAMPLES_EXT)) {
+        rasterization_samples = cb_state.dynamic_state_value.rasterization_samples;
+    } else {
+        const auto ms_state = pipeline_state->MultisampleState();
+        if (ms_state) {
+            rasterization_samples = ms_state->rasterizationSamples;
+        }
+    }
+    return rasterization_samples;
+}
+
+bool LAST_BOUND_STATE::IsRasterizationDisabled() const {
+    return pipeline_state->IsDynamic(VK_DYNAMIC_STATE_RASTERIZER_DISCARD_ENABLE_EXT)
+               ? cb_state.dynamic_state_value.rasterizer_discard_enable
+               : pipeline_state->RasterizationDisabled();
+}
+
+bool LAST_BOUND_STATE::ValidShaderObjectCombination(const VkPipelineBindPoint bind_point,
+                                                    const DeviceFeatures &device_features) const {
+    if (bind_point == VK_PIPELINE_BIND_POINT_COMPUTE) {
+        if (!IsValidShaderOrNullBound(ShaderObjectStage::COMPUTE) ||
+            !IsValidShaderOrNullBound(ShaderObjectStage::COMPUTE))
+            return false;
+    } else {
+        if (!IsValidShaderOrNullBound(ShaderObjectStage::VERTEX)) return false;
+        if (device_features.core.tessellationShader &&
+            !IsValidShaderOrNullBound(ShaderObjectStage::TESSELLATION_CONTROL))
+            return false;
+        if (device_features.core.tessellationShader &&
+            !IsValidShaderOrNullBound(ShaderObjectStage::TESSELLATION_EVALUATION))
+            return false;
+        if (device_features.core.geometryShader && !IsValidShaderOrNullBound(ShaderObjectStage::GEOMETRY))
+            return false;
+        if (!IsValidShaderOrNullBound(ShaderObjectStage::FRAGMENT)) return false;
+        if (device_features.mesh_shader_features.taskShader && !IsValidShaderOrNullBound(ShaderObjectStage::TASK))
+            return false;
+        if (device_features.mesh_shader_features.meshShader && !IsValidShaderOrNullBound(ShaderObjectStage::MESH))
+            return false;
+        if (GetShader(ShaderObjectStage::VERTEX) == VK_NULL_HANDLE &&
+            (!device_features.mesh_shader_features.meshShader ||
+             GetShader(ShaderObjectStage::MESH) == VK_NULL_HANDLE))
+            return false;
+    }
+    return true;
+}
+
+VkShaderEXT LAST_BOUND_STATE::GetShader(ShaderObjectStage stage) const {
+    if (!IsValidShaderBound(stage) || GetShaderState(stage) == nullptr) return VK_NULL_HANDLE;
+    return shader_object_states[static_cast<uint32_t>(stage)]->shader();
+}
+
+SHADER_OBJECT_STATE *LAST_BOUND_STATE::GetShaderState(ShaderObjectStage stage) const {
+    return shader_object_states[static_cast<uint32_t>(stage)];
+}
+
+bool LAST_BOUND_STATE::HasShaderObjects() const {
+    for (uint32_t i = 0; i < SHADER_OBJECT_STAGE_COUNT; ++i) {
+        if (GetShader(static_cast<ShaderObjectStage>(i)) != VK_NULL_HANDLE) {
+            return true;
+        }
+    }
+    return false;
+}
+
+bool LAST_BOUND_STATE::IsValidShaderBound(ShaderObjectStage stage) const {
+    if (!shader_object_bound[static_cast<uint32_t>(stage)]) {
+        return false;
+    }
+    return shader_object_states[static_cast<uint32_t>(stage)] != nullptr;
+}
+
+bool LAST_BOUND_STATE::IsValidShaderOrNullBound(ShaderObjectStage stage) const {
+    return shader_object_bound[static_cast<uint32_t>(stage)];
+}
