@@ -30,6 +30,7 @@
 #include "spirv-tools/optimizer.hpp"
 
 class PIPELINE_STATE;
+struct EntryPoint;
 
 // This is the common info for both OpDecorate and OpMemberDecorate
 // Used to keep track of all decorations applied to any instruction
@@ -57,6 +58,8 @@ struct DecorationBase {
     // Component is optional and spec says it is 0 if not defined
     uint32_t component = 0;
 
+    uint32_t offset = 0;
+
     // A given object can only have a single BuiltIn OpDecoration
     uint32_t builtin = kInvalidValue;
 
@@ -79,6 +82,7 @@ struct DecorationSet : public DecorationBase {
 
     void Add(uint32_t decoration, uint32_t value);
     bool HasBuiltIn() const;
+    bool HasInMember(FlagBit flag_bit) const;
     bool AllMemberHave(FlagBit flag_bit) const;
 };
 
@@ -136,11 +140,47 @@ struct TypeStructInfo {
     const uint32_t id;
     const uint32_t length;  // number of elements
     const DecorationSet &decorations;
-    std::vector<uint32_t> member_ids;
 
-    TypeStructInfo(const SHADER_MODULE_STATE &module_state, const Instruction &struct_insn);
+    // data about each member in struct
+    struct Member {
+        uint32_t id;
+        const Instruction *insn;
+        const DecorationBase *decorations;
+        std::shared_ptr<const TypeStructInfo> type_struct_info;  // for nested structs
+    };
+    std::vector<Member> members;
+
+    TypeStructInfo(const SPIRV_MODULE_STATE &module_state, const Instruction &struct_insn);
 };
 
+// Represents the OpImage* instructions and how it maps to the variable
+// This is created in the SPIRV_MODULE_STATE but then used with VariableBase objects
+struct ImageAccess {
+    const Instruction &image_insn;  // OpImage*
+    const Instruction *variable_image_insn = nullptr;
+    // If there is a OpSampledImage there will also be a sampler variable
+    const Instruction *variable_sampler_insn = nullptr;
+    bool no_function_jump = true;  // TODO 5614 - Handle function jumps
+
+    bool is_dref = false;
+    bool is_sampler_implicitLod_dref_proj = false;
+    bool is_sampler_sampled = false;
+    bool is_sampler_bias_offset = false;
+    bool is_written_to = false;
+    bool is_read_from = false;
+
+    static constexpr uint32_t kInvalidValue = std::numeric_limits<uint32_t>::max();
+    uint32_t image_access_chain_index = kInvalidValue;    // OpAccessChain's Index 0
+    uint32_t sampler_access_chain_index = kInvalidValue;  // OpAccessChain's Index 0
+    uint32_t texel_component_count = kInvalidValue;
+
+    ImageAccess(const SPIRV_MODULE_STATE &module_state, const Instruction &image_insn);
+};
+
+// <Image OpVariable Result ID, [ImageAccess, ImageAccess, etc] > - used for faster lookup
+// Many ImageAccess can point to a single Image Variable
+using ImageAccessMap = vvl::unordered_map<uint32_t, std::vector<std::shared_ptr<const ImageAccess>>>;
+
 // A slot is a <Location, Component> mapping
 struct InterfaceSlot {
     // A Location is made up of 4 Components
@@ -174,6 +214,16 @@ struct InterfaceSlot {
     uint32_t GetSlotValue(uint32_t location, uint32_t component) { return (location * 4) + component; }
 };
 
+// Represents the Image formats that can map to a SPIR-V format
+enum NumericType {
+    NumericTypeUnknown = 0,  // In case image is not used
+    NumericTypeFloat = 1,    // UNORM, SNORM, FLOAT, USCALED, SSCALED, SRGB -- anything we consider float in the shader
+    NumericTypeSint = 2,
+    NumericTypeUint = 4,
+};
+uint32_t GetFormatType(VkFormat format);
+char const *string_NumericType(uint32_t type);
+
 // Common info needed for all OpVariable
 struct VariableBase {
     const uint32_t id;
@@ -184,7 +234,7 @@ struct VariableBase {
     // If variable is accessed from mulitple entrypoint, create a seperate VariableBase object as info about it being access will be
     // different
     const VkShaderStageFlagBits stage;
-    VariableBase(const SHADER_MODULE_STATE &module_state, const Instruction &insn, VkShaderStageFlagBits stage);
+    VariableBase(const SPIRV_MODULE_STATE &module_state, const Instruction &insn, VkShaderStageFlagBits stage);
 };
 
 // These are Input/Output OpVariable that go in-between stages
@@ -201,23 +251,26 @@ struct StageInteraceVariable : public VariableBase {
     const bool is_per_task_nv;  // VK_NV_mesh_shader
 
     const bool is_array_interface;
+    uint32_t array_size = 1;  // flatten size of all dimensions; 1 if no array
     const Instruction &base_type;
     const bool is_builtin;
     bool nested_struct;
+    bool physical_storage_buffer;
 
     const std::vector<InterfaceSlot> interface_slots;  // Only for User Defined variables
     const std::vector<uint32_t> builtin_block;
     uint32_t total_builtin_components = 0;
 
-    StageInteraceVariable(const SHADER_MODULE_STATE &module_state, const Instruction &insn, VkShaderStageFlagBits stage);
+    StageInteraceVariable(const SPIRV_MODULE_STATE &module_state, const Instruction &insn, VkShaderStageFlagBits stage);
 
   protected:
+    static bool IsPerTaskNV(const StageInteraceVariable &variable);
     static bool IsArrayInterface(const StageInteraceVariable &variable);
-    static const Instruction &FindBaseType(const StageInteraceVariable &variable, const SHADER_MODULE_STATE &module_state);
-    static bool IsBuiltin(const StageInteraceVariable &variable, const SHADER_MODULE_STATE &module_state);
-    static std::vector<InterfaceSlot> GetInterfaceSlots(StageInteraceVariable &variable, const SHADER_MODULE_STATE &module_state);
-    static std::vector<uint32_t> GetBuiltinBlock(const StageInteraceVariable &variable, const SHADER_MODULE_STATE &module_state);
-    static uint32_t GetBuiltinComponents(const StageInteraceVariable &variable, const SHADER_MODULE_STATE &module_state);
+    static const Instruction &FindBaseType(StageInteraceVariable &variable, const SPIRV_MODULE_STATE &module_state);
+    static bool IsBuiltin(const StageInteraceVariable &variable, const SPIRV_MODULE_STATE &module_state);
+    static std::vector<InterfaceSlot> GetInterfaceSlots(StageInteraceVariable &variable, const SPIRV_MODULE_STATE &module_state);
+    static std::vector<uint32_t> GetBuiltinBlock(const StageInteraceVariable &variable, const SPIRV_MODULE_STATE &module_state);
+    static uint32_t GetBuiltinComponents(const StageInteraceVariable &variable, const SPIRV_MODULE_STATE &module_state);
 };
 
 // vkspec.html#interfaces-resources describes 'Shader Resource Interface'
@@ -228,26 +281,33 @@ struct StageInteraceVariable : public VariableBase {
 // at draw/submit time we can cross reference with the last bound descriptor.
 struct ResourceInterfaceVariable : public VariableBase {
     // If the type is a OpTypeArray save the length
-    uint32_t array_length = 0;
-    bool runtime_array = false;  // OpTypeRuntimeArray - can't validate length until run time
+    uint32_t array_length;
+    bool runtime_array;  // OpTypeRuntimeArray - can't validate length until run time
+
+    bool is_sampled_image;  // OpTypeSampledImage
 
     // List of samplers that sample a given image. The index of array is index of image.
     std::vector<vvl::unordered_set<SamplerUsedByImage>> samplers_used_by_image;
 
-    // For storage images - list of < OpImageWrite : Texel component length >
-    std::vector<std::pair<Instruction, uint32_t>> write_without_formats_component_count_list;
+    // For storage images - list of Texel component length the OpImageWrite
+    std::vector<uint32_t> write_without_formats_component_count_list;
 
     // A variable can have an array of indexes, need to track which are written to
     // can't use bitset because number of indexes isn't known until runtime
     // This array will match the OpTypeArray and not consider the InputAttachmentIndex
     std::vector<bool> input_attachment_index_read;
 
-    // Sampled Type width of the OpTypeImage the variable points to, 0 if doesn't use the image
-    uint32_t image_sampled_type_width = 0;
-
     // Type once array/pointer are stripped
     // most likly will be OpTypeImage, OpTypeStruct, OpTypeSampler, or OpTypeAccelerationStructureKHR
-    spv::Op base_type;
+    const Instruction &base_type;
+
+    const NumericType image_format_type;
+    bool IsImage() const { return image_format_type != NumericTypeUnknown; }
+    const spv::Dim image_dim;
+    const bool is_image_array;
+    const bool is_multisampled;
+    // Sampled Type width of the OpTypeImage the variable points to, 0 if doesn't use the image
+    uint32_t image_sampled_type_width = 0;
 
     bool is_read_from{false};   // has operation to reads from the variable
     bool is_written_to{false};  // has operation to writes to the variable
@@ -255,7 +315,7 @@ struct ResourceInterfaceVariable : public VariableBase {
     // Type of resource type (vkspec.html#interfaces-resources-storage-class-correspondence)
     bool is_storage_image{false};
     bool is_storage_texel_buffer{false};
-    bool is_storage_buffer{false};
+    const bool is_storage_buffer;
     bool is_input_attachment{false};
 
     bool is_atomic_operation{false};
@@ -264,128 +324,101 @@ struct ResourceInterfaceVariable : public VariableBase {
     bool is_sampler_bias_offset{false};
     bool is_read_without_format{false};   // For storage images
     bool is_write_without_format{false};  // For storage images
-    bool is_dref_operation{false};
+    bool is_dref{false};
 
-    ResourceInterfaceVariable(const SHADER_MODULE_STATE &module_state, const Instruction &insn, VkShaderStageFlagBits stage);
+    ResourceInterfaceVariable(const SPIRV_MODULE_STATE &module_state, const EntryPoint &entrypoint, const Instruction &insn,
+                              const ImageAccessMap &image_access_map);
+
+  protected:
+    static const Instruction &FindBaseType(ResourceInterfaceVariable &variable, const SPIRV_MODULE_STATE &module_state);
+    static NumericType FindImageFormatType(const SPIRV_MODULE_STATE &module_state, const Instruction &base_type);
+    static bool IsStorageBuffer(const ResourceInterfaceVariable &variable);
 };
 
-enum FORMAT_TYPE {
-    FORMAT_TYPE_FLOAT = 1,  // UNORM, SNORM, FLOAT, USCALED, SSCALED, SRGB -- anything we consider float in the shader
-    FORMAT_TYPE_SINT = 2,
-    FORMAT_TYPE_UINT = 4,
+// Used to help detect if different variable is being used
+inline bool operator==(const ResourceInterfaceVariable &a, const ResourceInterfaceVariable &b) noexcept { return a.id == b.id; }
+inline bool operator<(const ResourceInterfaceVariable &a, const ResourceInterfaceVariable &b) noexcept { return a.id < b.id; }
+
+// vkspec.html#interfaces-resources-pushconst
+// Push constants need to be statically used in shader
+// Push constants are always OpTypeStruct and Block decorated
+struct PushConstantVariable : public VariableBase {
+    // This info could be found/saved in TypeStructInfo, but since Push Constants are the only ones using it right now, no point to
+    // do it for every struct
+    uint32_t offset;  // where first member is
+    uint32_t size;    // total size of block
+
+    PushConstantVariable(const SPIRV_MODULE_STATE &module_state, const Instruction &insn, VkShaderStageFlagBits stage);
 };
 
-struct SHADER_MODULE_STATE : public BASE_NODE {
-    // Contains all the details for a OpTypeStruct
-    struct StructInfo {
-        uint32_t offset;
-        uint32_t size;                                 // A scalar size or a struct size. Not consider array
-        std::vector<uint32_t> array_length_hierarchy;  // multi-dimensional array, mat, vec. mat is combined with 2 array.
-                                                       // e.g :array[2] -> {2}, array[2][3][4] -> {2,3,4}, mat4[2] ->{2,4,4},
-        std::vector<uint32_t> array_block_size;        // When index increases, how many data increases.
-                                                 // e.g : array[2][3][4] -> {12,4,1}, it means if the first index increases one, the
-                                                 // array gets 12 data. If the second index increases one, the array gets 4 data.
+// Represents a single Entrypoint into a Shader Module
+struct EntryPoint {
+    // "A module must not have two OpEntryPoint instructions with the same Execution Model and the same Name string."
+    // There is no single unique item for a single entry point
+    const Instruction &entrypoint_insn;  // OpEntryPoint instruction
+    // For things like MeshNV vs MeshEXT, we need the execution_model
+    const spv::ExecutionModel execution_model;
+    const VkShaderStageFlagBits stage;
+    const uint32_t id;
+    const std::string name;
+    const ExecutionModeSet &execution_mode;
 
-        // OpTypeStruct can have OpTypeStruct inside it so need to track the struct-in-struct chain
-        std::vector<StructInfo> struct_members;  // If the data is not a struct, it's empty.
-        StructInfo *root;
+    // Values found while gather the Accessible Ids
+    bool emit_vertex_geometry;
 
-        StructInfo() : offset(0), size(0), root(nullptr) {}
+    // All ids that can be accessed from the entry point
+    const vvl::unordered_set<uint32_t> accessible_ids;
 
-        bool IsUsed() const {
-            if (!root) return false;
-            return !root->used_bytes.empty();
-        }
+    // only one Push Constant block is allowed per entry point
+    std::shared_ptr<const PushConstantVariable> push_constant_variable;
+    const std::vector<ResourceInterfaceVariable> resource_interface_variables;
+    const std::vector<StageInteraceVariable> stage_interface_variables;
+    // Easier to lookup without having to check for the is_builtin bool
+    // "Built-in interface variables" - vkspec.html#interfaces-iointerfaces-builtin
+    std::vector<const StageInteraceVariable *> built_in_variables;
+    // "User-defined Variable Interface" - vkspec.html#interfaces-iointerfaces-user
+    std::vector<const StageInteraceVariable *> user_defined_interface_variables;
 
-        std::vector<uint8_t> *GetUsedbytes() const {
-            if (!root) return nullptr;
-            return &root->used_bytes;
-        }
+    // Lookup map from Interface slot to the variable in that spot
+    // spirv-val guarantees no overlap so 2 variables won't have same slot
+    std::unordered_map<InterfaceSlot, const StageInteraceVariable *, InterfaceSlot::Hash> input_interface_slots;
+    std::unordered_map<InterfaceSlot, const StageInteraceVariable *, InterfaceSlot::Hash> output_interface_slots;
+    // Uesd for limit check
+    const StageInteraceVariable *max_input_slot_variable = nullptr;
+    const StageInteraceVariable *max_output_slot_variable = nullptr;
+    const InterfaceSlot *max_input_slot = nullptr;
+    const InterfaceSlot *max_output_slot = nullptr;
+    uint32_t builtin_input_components = 0;
+    uint32_t builtin_output_components = 0;
 
-        std::string GetLocationDesc(uint32_t index_used_bytes) const {
-            std::string desc = "";
-            if (array_length_hierarchy.size() > 0) {
-                desc += " index:";
-                for (const auto block_size : array_block_size) {
-                    desc += "[";
-                    desc += std::to_string(index_used_bytes / (block_size * size));
-                    desc += "]";
-                    index_used_bytes = index_used_bytes % (block_size * size);
-                }
-            }
-            const int struct_members_size = static_cast<int>(struct_members.size());
-            if (struct_members_size > 0) {
-                desc += " member:";
-                for (int i = struct_members_size - 1; i >= 0; --i) {
-                    if (index_used_bytes > struct_members[i].offset) {
-                        desc += std::to_string(i);
-                        desc += struct_members[i].GetLocationDesc(index_used_bytes - struct_members[i].offset);
-                        break;
-                    }
-                }
-            } else {
-                desc += " offset:";
-                desc += std::to_string(index_used_bytes);
-            }
-            return desc;
-        }
+    // Mark if a BuiltIn is written to
+    bool written_builtin_point_size{false};
+    bool written_builtin_primitive_shading_rate_khr{false};
+    bool written_builtin_viewport_index{false};
+    bool written_builtin_viewport_mask_nv{false};
 
-      private:
-        std::vector<uint8_t> used_bytes;  // This only works for root. 0: not used. 1: used. The totally array * size.
-    };
+    bool has_passthrough{false};
+    bool has_alpha_to_coverage_variable{false};  // only for Fragment shaders
 
-    struct EntryPoint {
-        // "A module must not have two OpEntryPoint instructions with the same Execution Model and the same Name string."
-        // There is no single unique item for a single entry point
-        const Instruction &entrypoint_insn;  // OpEntryPoint instruction
-        const VkShaderStageFlagBits stage;
-        const uint32_t id;
-        const std::string name;
-        // All ids that can be accessed from the entry point
-        vvl::unordered_set<uint32_t> accessible_ids;
+    EntryPoint(const SPIRV_MODULE_STATE &module_state, const Instruction &entrypoint_insn, const ImageAccessMap &image_access_map);
 
-        const ExecutionModeSet &execution_mode;
-
-        std::vector<ResourceInterfaceVariable> resource_interface_variables;
-        std::vector<StageInteraceVariable> stage_interface_variables;
-        // Easier to lookup without having to check for the is_builtin bool
-        // "Built-in interface variables" - vkspec.html#interfaces-iointerfaces-builtin
-        std::vector<const StageInteraceVariable *> built_in_variables;
-        // "User-defined Variable Interface" - vkspec.html#interfaces-iointerfaces-user
-        std::vector<const StageInteraceVariable *> user_defined_interface_variables;
-
-        // Lookup map from Interface slot to the variable in that spot
-        // spirv-val guarantees no overlap so 2 variables won't have same slot
-        std::unordered_map<InterfaceSlot, const StageInteraceVariable *, InterfaceSlot::Hash> input_interface_slots;
-        std::unordered_map<InterfaceSlot, const StageInteraceVariable *, InterfaceSlot::Hash> output_interface_slots;
-        // Uesd for limit check
-        const StageInteraceVariable *max_input_slot_variable = nullptr;
-        const StageInteraceVariable *max_output_slot_variable = nullptr;
-        const InterfaceSlot *max_input_slot = nullptr;
-        const InterfaceSlot *max_output_slot = nullptr;
-        uint32_t builtin_input_components = 0;
-        uint32_t builtin_output_components = 0;
-
-        StructInfo push_constant_used_in_shader;
-
-        bool emit_vertex_geometry{false};
-
-        // Mark if a BuiltIn is written to
-        bool written_builtin_point_size{false};
-        bool written_builtin_primitive_shading_rate_khr{false};
-        bool written_builtin_viewport_index{false};
-        bool written_builtin_viewport_mask_nv{false};
-
-        bool has_passthrough{false};
-
-        EntryPoint(const SHADER_MODULE_STATE &module_state, const Instruction &entrypoint_insn);
-    };
+  protected:
+    static vvl::unordered_set<uint32_t> GetAccessibleIds(const SPIRV_MODULE_STATE &module_state, EntryPoint &entrypoint);
+    static std::vector<StageInteraceVariable> GetStageInterfaceVariables(const SPIRV_MODULE_STATE &module_state,
+                                                                         const EntryPoint &entrypoint);
+    static std::vector<ResourceInterfaceVariable> GetResourceInterfaceVariables(const SPIRV_MODULE_STATE &module_state,
+                                                                                EntryPoint &entrypoint,
+                                                                                const ImageAccessMap &image_access_map);
+};
 
+// Represents a SPIR-V Module
+// This holds the SPIR-V source and parse it
+struct SPIRV_MODULE_STATE {
     // Static/const data extracted from a SPIRV module at initialization time
     // The goal of this struct is to move everything that is ready only into here
     struct StaticData {
         StaticData() = default;
-        StaticData(const SHADER_MODULE_STATE &module_state);
+        StaticData(const SPIRV_MODULE_STATE &module_state);
         StaticData &operator=(StaticData &&) = default;
         StaticData(StaticData &&) = default;
 
@@ -413,8 +446,14 @@ struct SHADER_MODULE_STATE : public BASE_NODE {
         std::vector<const Instruction *> variable_inst;
         // both OpDecorate and OpMemberDecorate builtin instructions
         std::vector<const Instruction *> builtin_decoration_inst;
-        // OpEmitStreamVertex/OpEndStreamPrimitive
+        // OpEmitStreamVertex/OpEndStreamPrimitive - only allowed in Geometry shader
         std::vector<const Instruction *> transform_feedback_stream_inst;
+        // OpString - used to find debug information
+        std::vector<const Instruction *> debug_string_inst;
+        // For shader tile image - OpDepthAttachmentReadEXT/OpStencilAttachmentReadEXT/OpColorAttachmentReadEXT
+        bool has_shader_tile_image_depth_read{false};
+        bool has_shader_tile_image_stencil_read{false};
+        bool has_shader_tile_image_color_read{false};
         // BuiltIn we just care about existing or not, don't have to be written to
         bool has_builtin_layer{false};
         bool has_builtin_fully_covered{false};
@@ -422,6 +461,8 @@ struct SHADER_MODULE_STATE : public BASE_NODE {
         uint32_t builtin_workgroup_size_id = 0;
 
         std::vector<const Instruction *> atomic_inst;
+        std::vector<const Instruction *> group_inst;
+        std::vector<const Instruction *> read_clock_inst;
         std::vector<spv::Capability> capability_list;
 
         bool has_specialization_constants{false};
@@ -438,17 +479,10 @@ struct SHADER_MODULE_STATE : public BASE_NODE {
 
         // Tracks accesses (load, store, atomic) to the instruction calling them
         // Example: the OpLoad does the "access" but need to know if a OpImageRead uses that OpLoad later
-        std::vector<uint32_t> image_read_load_ids;
-        std::vector<uint32_t> image_write_load_ids;
         vvl::unordered_map<const Instruction *, uint32_t> image_write_load_id_map;  // <OpImageWrite, load id>
         std::vector<uint32_t> atomic_pointer_ids;
         std::vector<uint32_t> store_pointer_ids;
         std::vector<uint32_t> atomic_store_pointer_ids;
-        std::vector<uint32_t> sampler_load_ids;  // tracks all sampling operations
-        std::vector<uint32_t> sampler_implicitLod_dref_proj_load_ids;
-        std::vector<uint32_t> sampler_bias_offset_load_ids;
-        std::vector<uint32_t> image_dref_load_ids;
-        std::vector<std::pair<uint32_t, uint32_t>> sampled_image_load_ids;                       // <image, sampler>
         vvl::unordered_map<uint32_t, uint32_t> load_members;                              // <result id, pointer>
         vvl::unordered_map<uint32_t, std::pair<uint32_t, uint32_t>> accesschain_members;  // <result id, <base,index[0]>>
         vvl::unordered_map<uint32_t, uint32_t> image_texel_pointer_members;               // <result id, image>
@@ -457,24 +491,17 @@ struct SHADER_MODULE_STATE : public BASE_NODE {
     // This is the SPIR-V module data content
     const std::vector<uint32_t> words_;
 
-    const bool has_valid_spirv{false};
     const StaticData static_data_;
 
-    uint32_t gpu_validation_shader_id{std::numeric_limits<uint32_t>::max()};
+    // Hold a handle so error message can know where the SPIR-V was from (VkShaderModule or VkShaderEXT)
+    VulkanTypedHandle handle_;                            // Will be updated once its known its valid SPIR-V
+    VulkanTypedHandle handle() const { return handle_; }  // matches normal convention to get handle
 
-    explicit SHADER_MODULE_STATE(vvl::span<const uint32_t> code)
-        : BASE_NODE(static_cast<VkShaderModule>(VK_NULL_HANDLE), kVulkanObjectTypeShaderModule),
-          words_(code.begin(), code.end()),
-          static_data_(*this) {}
+    // Used for when modifying the SPIR-V (spirv-opt, GPU-AV instrumentation, etc) and need reparse it for VVL validaiton
+    SPIRV_MODULE_STATE(vvl::span<const uint32_t> code) : words_(code.begin(), code.end()), static_data_(*this) {}
 
-    SHADER_MODULE_STATE(const VkShaderModuleCreateInfo &create_info, VkShaderModule shaderModule, uint32_t unique_shader_id)
-        : BASE_NODE(shaderModule, kVulkanObjectTypeShaderModule),
-          words_(create_info.pCode, create_info.pCode + create_info.codeSize / sizeof(uint32_t)),
-          has_valid_spirv(true),
-          static_data_(*this),
-          gpu_validation_shader_id(unique_shader_id) {}
-
-    SHADER_MODULE_STATE() : BASE_NODE(static_cast<VkShaderModule>(VK_NULL_HANDLE), kVulkanObjectTypeShaderModule) {}
+    SPIRV_MODULE_STATE(size_t codeSize, const uint32_t *pCode)
+        : words_(pCode, pCode + codeSize / sizeof(uint32_t)), static_data_(*this) {}
 
     const Instruction *FindDef(uint32_t id) const {
         auto it = static_data_.definitions.find(id);
@@ -487,10 +514,6 @@ struct SHADER_MODULE_STATE : public BASE_NODE {
     const std::vector<const Instruction *> FindVariableAccesses(uint32_t variable_id, const std::vector<uint32_t> &access_ids,
                                                                 bool atomic) const;
 
-    bool HasMultipleEntryPoints() const { return static_data_.entry_points.size() > 1; }
-
-    VkShaderModule vk_shader_module() const { return handle_.Cast<VkShaderModule>(); }
-
     const DecorationSet &GetDecorationSet(uint32_t id) const {
         // return the actual decorations for this id, or a default empty set.
         const auto it = static_data_.decorations.find(id);
@@ -515,7 +538,7 @@ struct SHADER_MODULE_STATE : public BASE_NODE {
                 insn = FindDef(insn->Word(1));
             } else if (insn->Opcode() == spv::OpTypePointer) {
                 insn = FindDef(insn->Word(3));
-            } else if (insn->Opcode() == spv::OpTypeArray || insn->Opcode() == spv::OpTypeRuntimeArray) {
+            } else if (insn->IsArray()) {
                 insn = FindDef(insn->Word(2));
             } else if (insn->Opcode() == spv::OpTypeStruct) {
                 return GetTypeStructInfo(insn->Word(1));
@@ -531,20 +554,16 @@ struct SHADER_MODULE_STATE : public BASE_NODE {
 
     std::optional<VkPrimitiveTopology> GetTopology(const EntryPoint &entrypoint) const;
 
-    const StructInfo *FindEntrypointPushConstant(char const *name, VkShaderStageFlagBits stageBits) const;
     std::shared_ptr<const EntryPoint> FindEntrypoint(char const *name, VkShaderStageFlagBits stageBits) const;
-    [[nodiscard]] bool FindLocalSize(const EntryPoint &entrypoint, uint32_t &local_size_x, uint32_t &local_size_y,
-                                     uint32_t &local_size_z) const;
+    bool FindLocalSize(const EntryPoint &entrypoint, uint32_t &local_size_x, uint32_t &local_size_y, uint32_t &local_size_z) const;
+
+    uint32_t CalculateWorkgroupSharedMemory() const;
 
     const Instruction *GetConstantDef(uint32_t id) const;
     uint32_t GetConstantValueById(uint32_t id) const;
-    spv::Dim GetShaderResourceDimensionality(const ResourceInterfaceVariable &resource) const;
     uint32_t GetLocationsConsumedByType(uint32_t type) const;
     uint32_t GetComponentsConsumedByType(uint32_t type) const;
-    uint32_t GetFundamentalType(uint32_t type) const;
-    const Instruction *GetStructType(const Instruction *insn) const;
-
-    uint32_t DescriptorTypeToReqs(uint32_t type_id) const;
+    NumericType GetNumericType(uint32_t type) const;
 
     bool IsBuiltInWritten(const Instruction *builtin_insn, const EntryPoint &entrypoint) const;
 
@@ -556,26 +575,29 @@ struct SHADER_MODULE_STATE : public BASE_NODE {
     const Instruction *GetBaseTypeInstruction(uint32_t type) const;
     uint32_t GetTypeId(uint32_t id) const;
     uint32_t GetTexelComponentCount(const Instruction &insn) const;
+    uint32_t GetFlattenArraySize(const Instruction &insn) const;
 
     bool HasCapability(spv::Capability find_capability) const {
         return std::any_of(static_data_.capability_list.begin(), static_data_.capability_list.end(),
                            [find_capability](const spv::Capability &capability) { return capability == find_capability; });
     }
+};
 
-    // Used to set push constants values at shader module initialization time
-    static void SetPushConstantUsedInShader(const SHADER_MODULE_STATE &module_state,
-                                            std::vector<std::shared_ptr<SHADER_MODULE_STATE::EntryPoint>> &entry_points);
+// Represents a VkShaderModule handle
+struct SHADER_MODULE_STATE : public BASE_NODE {
+    SHADER_MODULE_STATE(VkShaderModule shader_module, std::shared_ptr<SPIRV_MODULE_STATE> &spirv_module, uint32_t unique_shader_id)
+        : BASE_NODE(shader_module, kVulkanObjectTypeShaderModule), spirv(spirv_module), gpu_validation_shader_id(unique_shader_id) {
+        spirv->handle_ = handle_;
+    }
 
-  private:
-    // The following are all helper functions to set the push constants values by tracking if the values are accessed in the entry
-    // point functions and which offset in the structs are used
-    uint32_t UpdateOffset(uint32_t offset, const std::vector<uint32_t> &array_indices, const StructInfo &data) const;
-    void SetUsedBytes(uint32_t offset, const std::vector<uint32_t> &array_indices, const StructInfo &data) const;
-    void DefineStructMember(const Instruction *insn, StructInfo &data) const;
-    void RunUsedArray(uint32_t offset, std::vector<uint32_t> array_indices, uint32_t access_chain_word_index,
-                      const Instruction *access_chain, const StructInfo &data) const;
-    void RunUsedStruct(uint32_t offset, uint32_t access_chain_word_index, const Instruction *access_chain,
-                       const StructInfo &data) const;
-    void SetUsedStructMember(const uint32_t variable_id, vvl::unordered_set<uint32_t> &accessible_ids,
-                             const StructInfo &data) const;
+    // For when we need to create a module with no SPIR-V backing it
+    SHADER_MODULE_STATE(uint32_t unique_shader_id)
+        : BASE_NODE(static_cast<VkShaderModule>(VK_NULL_HANDLE), kVulkanObjectTypeShaderModule),
+          gpu_validation_shader_id(unique_shader_id) {}
+
+    // If null, means this is a empty object and no shader backing it
+    std::shared_ptr<SPIRV_MODULE_STATE> spirv;
+
+    // Used as way to match instrumented GPU-AV shader to a VkShaderModule handle
+    uint32_t gpu_validation_shader_id = 0;
 };
