@@ -24,25 +24,31 @@ namespace core_error {
 struct Location;
 }
 
+struct DeviceExtensions;
 struct SubresourceRangeErrorCodes;
+struct DeviceExtensions;
 
 namespace sync_vuid_maps {
 using core_error::Location;
 
 extern const std::map<VkPipelineStageFlags2KHR, std::string> kFeatureNameMap;
 
-const std::string &GetBadFeatureVUID(const Location &loc, VkPipelineStageFlags2KHR bit, bool sync2_enabled);
+const std::string &GetBadFeatureVUID(const Location &loc, VkPipelineStageFlags2 bit, const DeviceExtensions &device_extensions);
 
 const std::string &GetBadAccessFlagsVUID(const Location &loc, VkAccessFlags2KHR bit);
 
 const std::string &GetStageQueueCapVUID(const Location &loc, VkPipelineStageFlags2KHR bit);
 
 enum class QueueError {
-    kSrcOrDstMustBeIgnore = 0,
-    kSpecialOrIgnoreOnly,
-    kSrcAndDstValidOrSpecial,
-    kSrcAndDestMustBeIgnore,
-    kSrcAndDstBothValid,
+    kSrcNoExternalExt = 0,
+    kDstNoExternalExt,
+    kSrcNoForeignExt,
+    kDstNoForeignExt,
+    kSync1ConcurrentNoIgnored,
+    kSync1ConcurrentSrc,
+    kSync1ConcurrentDst,
+    kExclusiveSrc,
+    kExclusiveDst,
 };
 
 extern const std::map<QueueError, std::string> kQueueErrorSummary;
@@ -69,12 +75,10 @@ enum class ImageError {
     kBadSync2OldLayout,
     kBadSync2NewLayout,
     kNotColorAspect,
-    kNotColorAspectYcbcr,
     kBadMultiplanarAspect,
     kBadPlaneCount,
     kNotDepthOrStencilAspect,
     kNotDepthAndStencilAspect,
-    kNotSeparateDepthAndStencilAspect,
     kSeparateDepthWithStencilLayout,
     kSeparateStencilhWithDepthLayout,
     kRenderPassMismatch,
@@ -94,8 +98,7 @@ const SubresourceRangeErrorCodes &GetSubResourceVUIDs(const Location &loc);
 enum class SubmitError {
     kTimelineSemSmallValue,
     kSemAlreadySignalled,
-    kOldBinaryCannotBeSignalled,  // timeline semaphores not supported
-    kBinaryCannotBeSignalled,     // timeline semaphores supported
+    kBinaryCannotBeSignalled,
     kTimelineSemMaxDiff,
     kProtectedFeatureDisabled,
     kBadUnprotectedSubmit,
@@ -111,4 +114,10 @@ enum class SubmitError {
 
 const std::string &GetQueueSubmitVUID(const Location &loc, SubmitError error);
 
+enum class ShaderTileImageError { kShaderTileImageFeatureError, kShaderTileImageBarrierError };
+
+const std::string &GetShaderTileImageVUID(const Location &loc, ShaderTileImageError error);
+
+const std::string &GetAccessMaskRayQueryVUIDSelector(const Location &loc, const DeviceExtensions &device_extensions);
+
 }  // namespace sync_vuid_maps
