@@ -19,11 +19,7 @@
 #include "binding.h"
 
 #include <string.h>  // memset(), memcmp()
-#include <algorithm>
 #include <cassert>
-#include <iostream>
-#include <vector>
-
 #include "generated/vk_typemap_helper.h"
 #include "generated/vk_format_utils.h"
 
@@ -898,6 +894,71 @@ VkResult ShaderModule::init_try(const Device &dev, const VkShaderModuleCreateInf
     return err;
 }
 
+NON_DISPATCHABLE_HANDLE_DTOR(Shader, vk::DestroyShaderEXT)
+
+void Shader::init(const Device &dev, const VkShaderCreateInfoEXT &info) {
+    NON_DISPATCHABLE_HANDLE_INIT(vk::CreateShadersEXT, dev, 1u, &info);
+}
+
+VkResult Shader::init_try(const Device &dev, const VkShaderCreateInfoEXT &info) {
+    VkShaderEXT mod;
+
+    VkResult err = vk::CreateShadersEXT(dev.handle(), 1u, &info, NULL, &mod);
+    if (err == VK_SUCCESS) NonDispHandle::init(dev.handle(), mod);
+
+    return err;
+}
+
+Shader::Shader(const Device &dev, const VkShaderStageFlagBits stage, const std::vector<uint32_t> &spv,
+               const VkDescriptorSetLayout *descriptorSetLayout, const VkPushConstantRange* pushConstRange) {
+    VkShaderCreateInfoEXT createInfo = LvlInitStruct<VkShaderCreateInfoEXT>();
+    createInfo.stage = stage;
+    createInfo.codeType = VK_SHADER_CODE_TYPE_SPIRV_EXT;
+    createInfo.codeSize = spv.size() * sizeof(spv[0]);
+    createInfo.pCode = spv.data();
+    createInfo.pName = "main";
+    if (descriptorSetLayout) {
+        createInfo.setLayoutCount = 1u;
+        createInfo.pSetLayouts = descriptorSetLayout;
+    }
+    if (pushConstRange) {
+        createInfo.pushConstantRangeCount = 1u;
+        createInfo.pPushConstantRanges = pushConstRange;
+    }
+    init(dev, createInfo);
+}
+
+Shader::Shader(const Device &dev, const VkShaderStageFlagBits stage, const std::vector<uint8_t> &binary,
+               const VkDescriptorSetLayout *descriptorSetLayout, const VkPushConstantRange *pushConstRange) {
+    VkShaderCreateInfoEXT createInfo = LvlInitStruct<VkShaderCreateInfoEXT>();
+    createInfo.stage = stage;
+    createInfo.codeType = VK_SHADER_CODE_TYPE_BINARY_EXT;
+    createInfo.codeSize = binary.size();
+    createInfo.pCode = binary.data();
+    createInfo.pName = "main";
+    if (descriptorSetLayout) {
+        createInfo.setLayoutCount = 1u;
+        createInfo.pSetLayouts = descriptorSetLayout;
+    }
+    if (pushConstRange) {
+        createInfo.pushConstantRangeCount = 1u;
+        createInfo.pPushConstantRanges = pushConstRange;
+    }
+    init(dev, createInfo);
+}
+
+Shader::Shader(const Device &dev, const VkShaderStageFlagBits stage, const std::vector<uint32_t> &spv,
+               VkShaderCreateFlagsEXT flags) {
+    VkShaderCreateInfoEXT createInfo = LvlInitStruct<VkShaderCreateInfoEXT>();
+    createInfo.flags = flags;
+    createInfo.stage = stage;
+    createInfo.codeType = VK_SHADER_CODE_TYPE_SPIRV_EXT;
+    createInfo.codeSize = spv.size() * sizeof(spv[0]);
+    createInfo.pCode = spv.data();
+    createInfo.pName = "main";
+    init(dev, createInfo);
+}
+
 NON_DISPATCHABLE_HANDLE_DTOR(Pipeline, vk::DestroyPipeline)
 
 void Pipeline::init(const Device &dev, const VkGraphicsPipelineCreateInfo &info) {
