@@ -1,9 +1,6 @@
 /*
- * Copyright (c) 2015-2023 The Khronos Group Inc.
- * Copyright (c) 2015-2023 Valve Corporation
- * Copyright (c) 2015-2023 LunarG, Inc.
- * Copyright (c) 2015-2023 Google, Inc.
- * Modifications Copyright (C) 2020 Advanced Micro Devices, Inc. All rights reserved.
+ * Copyright (c) 2023 Valve Corporation
+ * Copyright (c) 2023 LunarG, Inc.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -19,7 +16,8 @@
 namespace rt {
 namespace as {
 
-GeometryKHR::GeometryKHR(uint32_t vk_api_version) : vk_api_version_(vk_api_version), vk_obj_(LvlInitStruct<decltype(vk_obj_)>()) {}
+GeometryKHR::GeometryKHR(APIVersion vk_api_version)
+    : vk_api_version_(vk_api_version), vk_obj_(LvlInitStruct<decltype(vk_obj_)>()) {}
 
 GeometryKHR &GeometryKHR::SetType(Type type) {
     type_ = type;
@@ -148,9 +146,9 @@ GeometryKHR &GeometryKHR::SetInstanceDeviceAccelStructRef(const vk_testing::Devi
     auto alloc_flags = LvlInitStruct<VkMemoryAllocateFlagsInfo>();
     alloc_flags.flags = VK_MEMORY_ALLOCATE_DEVICE_ADDRESS_BIT_KHR;
     instance_.buffer.init(
-        device, sizeof(VkAccelerationStructureInstanceKHR), VK_MEMORY_PROPERTY_DEVICE_LOCAL_BIT,
+        device, sizeof(VkAccelerationStructureInstanceKHR),
         VK_BUFFER_USAGE_SHADER_DEVICE_ADDRESS_BIT | VK_BUFFER_USAGE_ACCELERATION_STRUCTURE_BUILD_INPUT_READ_ONLY_BIT_KHR,
-        &alloc_flags);
+        VK_MEMORY_PROPERTY_DEVICE_LOCAL_BIT, &alloc_flags);
 
     vk_obj_.geometry.instances.arrayOfPointers = VK_FALSE;
     vk_obj_.geometry.instances.data.deviceAddress = instance_.buffer.address(vk_api_version_);
@@ -179,7 +177,7 @@ VkAccelerationStructureBuildRangeInfoKHR GeometryKHR::GetFullBuildRange() const
     return range_info;
 }
 
-AccelerationStructureKHR::AccelerationStructureKHR(uint32_t vk_api_version)
+AccelerationStructureKHR::AccelerationStructureKHR(APIVersion vk_api_version)
     : vk_api_version_(vk_api_version), vk_info_(LvlInitStruct<decltype(vk_info_)>()), device_buffer_() {}
 
 AccelerationStructureKHR &AccelerationStructureKHR::SetSize(VkDeviceSize size) {
@@ -243,23 +241,20 @@ void AccelerationStructureKHR::Build(const vk_testing::Device &device) {
     if (!device_buffer_.initialized() && (buffer_usage_flags_ != 0)) {
         auto alloc_flags = LvlInitStruct<VkMemoryAllocateFlagsInfo>();
         alloc_flags.flags = buffer_memory_allocate_flags_;
+        auto ci = LvlInitStruct<VkBufferCreateInfo>();
+        ci.size = vk_info_.size;
+        ci.usage = buffer_usage_flags_;
         if (buffer_init_no_mem_) {
-            auto ci = LvlInitStruct<VkBufferCreateInfo>();
-            ci.size = vk_info_.size;
-            ci.usage = buffer_usage_flags_;
             device_buffer_.init_no_mem(device, ci);
         } else {
-            device_buffer_.init(device, vk_info_.size, buffer_memory_property_flags_, buffer_usage_flags_, &alloc_flags);
+            device_buffer_.init(device, ci, buffer_memory_property_flags_, &alloc_flags);
         }
     }
     vk_info_.buffer = device_buffer_.handle();
 
     // Create acceleration structure
-    auto vkCreateAccelerationStructureKHR = reinterpret_cast<PFN_vkCreateAccelerationStructureKHR>(
-        vk::GetDeviceProcAddr(device.handle(), "vkCreateAccelerationStructureKHR"));
-    assert(vkCreateAccelerationStructureKHR);
     VkAccelerationStructureKHR handle;
-    const VkResult result = vkCreateAccelerationStructureKHR(device.handle(), &vk_info_, nullptr, &handle);
+    const VkResult result = vk::CreateAccelerationStructureKHR(device.handle(), &vk_info_, nullptr, &handle);
     assert(result == VK_SUCCESS);
     if (result == VK_SUCCESS) {
         init(device.handle(), handle);
@@ -272,21 +267,18 @@ void AccelerationStructureKHR::Destroy() {
     }
     assert(device() != VK_NULL_HANDLE);
     assert(handle() != VK_NULL_HANDLE);
-    auto vkDestroyAccelerationStructureKHR = reinterpret_cast<PFN_vkDestroyAccelerationStructureKHR>(
-        vk::GetDeviceProcAddr(device(), "vkDestroyAccelerationStructureKHR"));
-    assert(vkDestroyAccelerationStructureKHR);
-    vkDestroyAccelerationStructureKHR(device(), handle(), nullptr);
+    vk::DestroyAccelerationStructureKHR(device(), handle(), nullptr);
     handle_ = VK_NULL_HANDLE;
     device_buffer_.destroy();
 }
 
-BuildGeometryInfoKHR::BuildGeometryInfoKHR(uint32_t vk_api_version)
+BuildGeometryInfoKHR::BuildGeometryInfoKHR(APIVersion vk_api_version)
     : vk_api_version_(vk_api_version),
       vk_info_(LvlInitStruct<decltype(vk_info_)>()),
       geometries_(),
       src_as_(std::make_shared<AccelerationStructureKHR>(vk_api_version)),
       dst_as_(std::make_shared<AccelerationStructureKHR>(vk_api_version)),
-      device_scratch_() {}
+      device_scratch_(std::make_shared<vk_testing::Buffer>()) {}
 
 BuildGeometryInfoKHR &BuildGeometryInfoKHR::SetGeometries(std::vector<GeometryKHR> &&geometries) {
     geometries_ = std::move(geometries);
@@ -327,18 +319,23 @@ BuildGeometryInfoKHR &BuildGeometryInfoKHR::SetDstAS(std::shared_ptr<Acceleratio
     return *this;
 }
 
-BuildGeometryInfoKHR &BuildGeometryInfoKHR::SetScratchBuffer(vk_testing::Buffer &&scratch_buffer) {
+BuildGeometryInfoKHR &BuildGeometryInfoKHR::SetScratchBuffer(std::shared_ptr<vk_testing::Buffer> scratch_buffer) {
     device_scratch_ = std::move(scratch_buffer);
     return *this;
 }
 
+BuildGeometryInfoKHR &BuildGeometryInfoKHR::SetDeviceScratchOffset(VkDeviceAddress offset) {
+    device_scratch_offset_ = offset;
+    return *this;
+}
+
 BuildGeometryInfoKHR &BuildGeometryInfoKHR::SetBottomLevelAS(std::shared_ptr<BuildGeometryInfoKHR> bottom_level_as) {
-    bottom_level_as_ = std::move(bottom_level_as);
+    blas_ = std::move(bottom_level_as);
     return *this;
 }
 
 BuildGeometryInfoKHR &BuildGeometryInfoKHR::SetInfoCount(uint32_t info_count) {
-    assert(info_count <= 1);  // TODO - support array of VkAccelerationStructureBuildGeometryInfoKHR
+    assert(info_count <= 1);
     vk_info_count_ = info_count;
     return *this;
 }
@@ -353,29 +350,28 @@ BuildGeometryInfoKHR &BuildGeometryInfoKHR::SetNullBuildRangeInfos(bool use_null
     return *this;
 }
 
-void BuildGeometryInfoKHR::BuildCmdBuffer(VkInstance instance, const vk_testing::Device &device, VkCommandBuffer cmd_buffer,
+void BuildGeometryInfoKHR::BuildCmdBuffer(const vk_testing::Device &device, VkCommandBuffer cmd_buffer,
                                           bool use_ppGeometries /*= true*/) {
-    if (bottom_level_as_) {
-        bottom_level_as_->BuildCmdBuffer(instance, device, cmd_buffer, use_ppGeometries);
+    if (blas_) {
+        blas_->BuildCmdBuffer(device, cmd_buffer, use_ppGeometries);
     }
-    BuildCommon(instance, device, true);
+    BuildCommon(device, true);
     VkCmdBuildAccelerationStructuresKHR(device, cmd_buffer, true);
 }
 
-void BuildGeometryInfoKHR::BuildCmdBufferIndirect(VkInstance instance, const vk_testing::Device &device,
-                                                  VkCommandBuffer cmd_buffer) {
-    if (bottom_level_as_) {
-        bottom_level_as_->BuildCmdBufferIndirect(instance, device, cmd_buffer);
+void BuildGeometryInfoKHR::BuildCmdBufferIndirect(const vk_testing::Device &device, VkCommandBuffer cmd_buffer) {
+    if (blas_) {
+        blas_->BuildCmdBufferIndirect(device, cmd_buffer);
     }
-    BuildCommon(instance, device, true);
+    BuildCommon(device, true);
     VkCmdBuildAccelerationStructuresIndirectKHR(device, cmd_buffer);
 }
 
 void BuildGeometryInfoKHR::BuildHost(VkInstance instance, const vk_testing::Device &device) {
-    if (bottom_level_as_) {
-        bottom_level_as_->BuildHost(instance, device);
+    if (blas_) {
+        blas_->BuildHost(instance, device);
     }
-    BuildCommon(instance, device, false);
+    BuildCommon(device, false);
     VkBuildAccelerationStructuresKHR(instance, device);
 }
 
@@ -391,7 +387,6 @@ void BuildGeometryInfoKHR::VkCmdBuildAccelerationStructuresKHR(const vk_testing:
     }
     std::vector<VkAccelerationStructureBuildRangeInfoKHR> range_infos(geometries_.size());
     std::vector<const VkAccelerationStructureBuildRangeInfoKHR *> pRange_infos(geometries_.size());
-    pGeometries.reserve(geometries_.size());
     for (size_t i = 0; i < geometries_.size(); ++i) {
         const auto &geometry = geometries_[i];
         if (use_ppGeometries) {
@@ -410,15 +405,13 @@ void BuildGeometryInfoKHR::VkCmdBuildAccelerationStructuresKHR(const vk_testing:
     }
 
     // Build acceleration structure
-    auto vkCmdBuildAccelerationStructuresKHR = reinterpret_cast<PFN_vkCmdBuildAccelerationStructuresKHR>(
-        vk::GetDeviceProcAddr(device.handle(), "vkCmdBuildAccelerationStructuresKHR"));
-    assert(vkCmdBuildAccelerationStructuresKHR);
     const VkAccelerationStructureBuildGeometryInfoKHR *pInfos = use_null_infos_ ? nullptr : &vk_info_;
     const VkAccelerationStructureBuildRangeInfoKHR *const *ppBuildRangeInfos =
         use_null_build_range_infos_ ? nullptr : pRange_infos.data();
-    vkCmdBuildAccelerationStructuresKHR(cmd_buffer, vk_info_count_, pInfos, ppBuildRangeInfos);
+    vk::CmdBuildAccelerationStructuresKHR(cmd_buffer, vk_info_count_, pInfos, ppBuildRangeInfos);
 
     // pGeometries and geometries are going to be destroyed
+    vk_info_.geometryCount = 0;
     vk_info_.ppGeometries = nullptr;
     vk_info_.pGeometries = nullptr;
 }
@@ -436,18 +429,15 @@ void BuildGeometryInfoKHR::VkCmdBuildAccelerationStructuresIndirectKHR(const vk_
     vk_info_.geometryCount = static_cast<uint32_t>(geometries_.size());
     vk_info_.ppGeometries = pGeometries.data();
 
-    auto vkCmdBuildAccelerationStructuresIndirectKHR = reinterpret_cast<PFN_vkCmdBuildAccelerationStructuresIndirectKHR>(
-        vk::GetDeviceProcAddr(device.handle(), "vkCmdBuildAccelerationStructuresIndirectKHR"));
-    assert(vkCmdBuildAccelerationStructuresIndirectKHR);
-
     VkDeviceAddress indirect_device_addresses{};
     uint32_t indirect_strides = sizeof(VkAccelerationStructureBuildRangeInfoKHR);
     uint32_t max_prim_counts[1] = {1};
 
-    vkCmdBuildAccelerationStructuresIndirectKHR(cmd_buffer, vk_info_count_, &vk_info_, &indirect_device_addresses,
-                                                &indirect_strides, reinterpret_cast<uint32_t **>(&max_prim_counts));
+    vk::CmdBuildAccelerationStructuresIndirectKHR(cmd_buffer, vk_info_count_, &vk_info_, &indirect_device_addresses,
+                                                  &indirect_strides, reinterpret_cast<uint32_t **>(&max_prim_counts));
 
     // pGeometries and geometries are going to be destroyed
+    vk_info_.geometryCount = 0;
     vk_info_.ppGeometries = nullptr;
     vk_info_.pGeometries = nullptr;
 }
@@ -468,15 +458,13 @@ void BuildGeometryInfoKHR::VkBuildAccelerationStructuresKHR(VkInstance instance,
     vk_info_.ppGeometries = pGeometries.data();
 
     // Build acceleration structure
-    auto vkBuildAccelerationStructuresKHR = reinterpret_cast<PFN_vkBuildAccelerationStructuresKHR>(
-        vk::GetInstanceProcAddr(instance, "vkBuildAccelerationStructuresKHR"));
-    assert(vkBuildAccelerationStructuresKHR);
     const VkAccelerationStructureBuildGeometryInfoKHR *pInfos = use_null_infos_ ? nullptr : &vk_info_;
     const VkAccelerationStructureBuildRangeInfoKHR *const *ppBuildRangeInfos =
         use_null_build_range_infos_ ? nullptr : pRange_infos.data();
-    vkBuildAccelerationStructuresKHR(device.handle(), VK_NULL_HANDLE, vk_info_count_, pInfos, ppBuildRangeInfos);
+    vk::BuildAccelerationStructuresKHR(device.handle(), VK_NULL_HANDLE, vk_info_count_, pInfos, ppBuildRangeInfos);
 
     // pGeometries is going to be destroyed
+    vk_info_.geometryCount = 0;
     vk_info_.ppGeometries = nullptr;
 }
 
@@ -506,21 +494,19 @@ VkAccelerationStructureBuildSizesInfoKHR BuildGeometryInfoKHR::GetSizeInfo(VkDev
     }
 
     // Get VkAccelerationStructureBuildSizesInfoKHR using this->vk_info_
-    auto vkGetAccelerationStructureBuildSizesKHR = reinterpret_cast<PFN_vkGetAccelerationStructureBuildSizesKHR>(
-        vk::GetDeviceProcAddr(device, "vkGetAccelerationStructureBuildSizesKHR"));
-    assert(vkGetAccelerationStructureBuildSizesKHR);
     auto size_info = LvlInitStruct<VkAccelerationStructureBuildSizesInfoKHR>();
-    vkGetAccelerationStructureBuildSizesKHR(device, VK_ACCELERATION_STRUCTURE_BUILD_TYPE_DEVICE_KHR, &vk_info_, &primitives_count,
-                                            &size_info);
+    vk::GetAccelerationStructureBuildSizesKHR(device, VK_ACCELERATION_STRUCTURE_BUILD_TYPE_DEVICE_KHR, &vk_info_, &primitives_count,
+                                              &size_info);
 
     // pGeometries and geometries are going to be destroyed
+    vk_info_.geometryCount = 0;
     vk_info_.ppGeometries = nullptr;
     vk_info_.pGeometries = nullptr;
 
     return size_info;
 }
 
-void BuildGeometryInfoKHR::BuildCommon(VkInstance instance, const vk_testing::Device &device, bool is_on_device_build,
+void BuildGeometryInfoKHR::BuildCommon(const vk_testing::Device &device, bool is_on_device_build,
                                        bool use_ppGeometries /*= true*/) {
     assert(vk_info_.mode == VK_BUILD_ACCELERATION_STRUCTURE_MODE_BUILD_KHR ||
            vk_info_.mode == VK_BUILD_ACCELERATION_STRUCTURE_MODE_UPDATE_KHR);
@@ -547,31 +533,32 @@ void BuildGeometryInfoKHR::BuildCommon(VkInstance instance, const vk_testing::De
         // Allocate device local scratch buffer
 
         // Get minAccelerationStructureScratchOffsetAlignment
-        auto vkGetPhysicalDeviceProperties2 = reinterpret_cast<PFN_vkGetPhysicalDeviceProperties2>(
-            vk::GetInstanceProcAddr(instance, "vkGetPhysicalDeviceProperties2"));
-        assert(vkGetPhysicalDeviceProperties2);
         auto as_props = LvlInitStruct<VkPhysicalDeviceAccelerationStructurePropertiesKHR>();
         auto phys_dev_props = LvlInitStruct<VkPhysicalDeviceProperties2>(&as_props);
-        vkGetPhysicalDeviceProperties2(device.phy(), &phys_dev_props);
+        vk::GetPhysicalDeviceProperties2(device.phy(), &phys_dev_props);
 
-        if (!device_scratch_.initialized()) {
+        assert(device_scratch_);  // So far null pointers are not supported
+        if (!device_scratch_->initialized()) {
             auto alloc_flags = LvlInitStruct<VkMemoryAllocateFlagsInfo>();
             alloc_flags.flags = VK_MEMORY_ALLOCATE_DEVICE_ADDRESS_BIT_KHR;
 
             if (scratch_size > 0) {
-                device_scratch_.init(device, scratch_size + as_props.minAccelerationStructureScratchOffsetAlignment,
-                                     VK_MEMORY_PROPERTY_DEVICE_LOCAL_BIT,
-                                     VK_BUFFER_USAGE_SHADER_DEVICE_ADDRESS_BIT | VK_BUFFER_USAGE_STORAGE_BUFFER_BIT, &alloc_flags);
+                device_scratch_->init(device, scratch_size + as_props.minAccelerationStructureScratchOffsetAlignment,
+                                      VK_BUFFER_USAGE_SHADER_DEVICE_ADDRESS_BIT | VK_BUFFER_USAGE_STORAGE_BUFFER_BIT,
+                                      VK_MEMORY_PROPERTY_DEVICE_LOCAL_BIT, &alloc_flags);
             }
         }
-        if (device_scratch_.create_info().size != 0 &&
-            device_scratch_.create_info().usage & VK_BUFFER_USAGE_SHADER_DEVICE_ADDRESS_BIT) {
-            const VkDeviceAddress scratch_address = device_scratch_.address(vk_api_version_);
+        if (device_scratch_->create_info().size != 0 &&
+            device_scratch_->create_info().usage & VK_BUFFER_USAGE_SHADER_DEVICE_ADDRESS_BIT) {
+            const VkDeviceAddress scratch_address = device_scratch_->address(vk_api_version_);
             const auto aligned_scratch_address =
                 Align<VkDeviceAddress>(scratch_address, as_props.minAccelerationStructureScratchOffsetAlignment);
             assert(aligned_scratch_address >= scratch_address);
             assert(aligned_scratch_address < (scratch_address + as_props.minAccelerationStructureScratchOffsetAlignment));
-            vk_info_.scratchData.deviceAddress = aligned_scratch_address;
+            vk_info_.scratchData.deviceAddress = aligned_scratch_address + device_scratch_offset_;
+            assert(vk_info_.scratchData.deviceAddress <
+                   (scratch_address +
+                    device_scratch_->create_info().size));  // Note: This assert may prove overly conservative in the future
         } else {
             vk_info_.scratchData.deviceAddress = 0;
         }
@@ -579,14 +566,71 @@ void BuildGeometryInfoKHR::BuildCommon(VkInstance instance, const vk_testing::De
         // Allocate on host scratch buffer
         host_scratch_ = nullptr;
         if (scratch_size > 0) {
-            host_scratch_ = std::make_unique<uint8_t[]>(scratch_size);
+            assert(scratch_size < vvl::kU32Max);
+            host_scratch_ = std::make_unique<uint8_t[]>(static_cast<size_t>(scratch_size));
         }
         vk_info_.scratchData.hostAddress = host_scratch_.get();
     }
 }
 
+void BuildAccelerationStructuresKHR(const vk_testing::Device &device, VkCommandBuffer cmd_buffer,
+                                    std::vector<BuildGeometryInfoKHR> &infos) {
+    size_t total_geomertry_count = 0;
+
+    for (auto &build_info : infos) {
+        total_geomertry_count += build_info.geometries_.size();
+    }
+
+    // Those vectors will be used to contiguously store the "raw vulkan data" for each element of `infos`
+    // To do that, total memory needed needs to be know upfront
+    std::vector<const VkAccelerationStructureGeometryKHR *> pGeometries(total_geomertry_count);
+    std::vector<VkAccelerationStructureBuildRangeInfoKHR> range_infos(total_geomertry_count);
+    std::vector<const VkAccelerationStructureBuildRangeInfoKHR *> pRange_infos(total_geomertry_count);
+
+    std::vector<VkAccelerationStructureBuildGeometryInfoKHR> vk_infos;
+    vk_infos.reserve(infos.size());
+
+    size_t pGeometries_offset = 0;
+    size_t range_infos_offset = 0;
+    size_t pRange_infos_offset = 0;
+
+    for (auto &build_info : infos) {
+        if (build_info.blas_) {
+            build_info.blas_->BuildCmdBuffer(device, cmd_buffer, true);
+        }
+        build_info.BuildCommon(device, true);
+
+        // Fill current vk_info_ with geometry data in ppGeometries, and get build ranges
+        for (size_t i = 0; i < build_info.geometries_.size(); ++i) {
+            const auto &geometry = build_info.geometries_[i];
+            pGeometries[pGeometries_offset + i] = &geometry.GetVkObj();
+            range_infos[range_infos_offset + i] = geometry.GetFullBuildRange();
+            pRange_infos[pRange_infos_offset + i] = &range_infos[range_infos_offset + i];
+        }
+
+        build_info.vk_info_.geometryCount = static_cast<uint32_t>(build_info.geometries_.size());
+        build_info.vk_info_.ppGeometries = &pGeometries[pGeometries_offset];
+
+        vk_infos.emplace_back(build_info.vk_info_);
+
+        pGeometries_offset += build_info.geometries_.size();
+        range_infos_offset += build_info.geometries_.size();
+        pRange_infos_offset += build_info.geometries_.size();
+    }
+
+    // Build list of acceleration structures
+    vk::CmdBuildAccelerationStructuresKHR(cmd_buffer, static_cast<uint32_t>(vk_infos.size()), vk_infos.data(), pRange_infos.data());
+
+    // Clean
+    for (auto &build_info : infos) {
+        // pGeometries is going to be destroyed
+        build_info.vk_info_.geometryCount = 0;
+        build_info.vk_info_.ppGeometries = nullptr;
+    }
+}
+
 namespace blueprint {
-GeometryKHR GeometrySimpleOnDeviceTriangleInfo(uint32_t vk_api_version, const vk_testing::Device &device) {
+GeometryKHR GeometrySimpleOnDeviceTriangleInfo(APIVersion vk_api_version, const vk_testing::Device &device) {
     GeometryKHR triangle_geometry(vk_api_version);
 
     triangle_geometry.SetType(GeometryKHR::Type::Triangle);
@@ -598,10 +642,10 @@ GeometryKHR GeometrySimpleOnDeviceTriangleInfo(uint32_t vk_api_version, const vk
                                             VK_BUFFER_USAGE_ACCELERATION_STRUCTURE_STORAGE_BIT_KHR |
                                             VK_BUFFER_USAGE_SHADER_DEVICE_ADDRESS_BIT;
 
-    vk_testing::Buffer vertex_buffer(device, 1024, VK_MEMORY_PROPERTY_HOST_VISIBLE_BIT | VK_MEMORY_PROPERTY_HOST_COHERENT_BIT,
-                                     buffer_usage, &alloc_flags);
-    vk_testing::Buffer index_buffer(device, 1024, VK_MEMORY_PROPERTY_HOST_VISIBLE_BIT | VK_MEMORY_PROPERTY_HOST_COHERENT_BIT,
-                                    buffer_usage, &alloc_flags);
+    vk_testing::Buffer vertex_buffer(device, 1024, buffer_usage,
+                                     VK_MEMORY_PROPERTY_HOST_VISIBLE_BIT | VK_MEMORY_PROPERTY_HOST_COHERENT_BIT, &alloc_flags);
+    vk_testing::Buffer index_buffer(device, 1024, buffer_usage,
+                                    VK_MEMORY_PROPERTY_HOST_VISIBLE_BIT | VK_MEMORY_PROPERTY_HOST_COHERENT_BIT, &alloc_flags);
 
     // Fill vertex and index buffers with one triangle
     triangle_geometry.SetPrimitiveCount(1);
@@ -623,7 +667,7 @@ GeometryKHR GeometrySimpleOnDeviceTriangleInfo(uint32_t vk_api_version, const vk
     return triangle_geometry;
 }
 
-GeometryKHR GeometrySimpleOnHostTriangleInfo(uint32_t vk_api_version) {
+GeometryKHR GeometrySimpleOnHostTriangleInfo(APIVersion vk_api_version) {
     GeometryKHR triangle_geometry(vk_api_version);
 
     triangle_geometry.SetType(GeometryKHR::Type::Triangle);
@@ -646,7 +690,7 @@ GeometryKHR GeometrySimpleOnHostTriangleInfo(uint32_t vk_api_version) {
     return triangle_geometry;
 }
 
-GeometryKHR GeometrySimpleOnDeviceAABBInfo(uint32_t vk_api_version, const vk_testing::Device &device) {
+GeometryKHR GeometrySimpleOnDeviceAABBInfo(APIVersion vk_api_version, const vk_testing::Device &device) {
     GeometryKHR aabb_geometry(vk_api_version);
 
     aabb_geometry.SetType(GeometryKHR::Type::AABB);
@@ -662,8 +706,8 @@ GeometryKHR GeometrySimpleOnDeviceAABBInfo(uint32_t vk_api_version, const vk_tes
                                             VK_BUFFER_USAGE_ACCELERATION_STRUCTURE_STORAGE_BIT_KHR |
                                             VK_BUFFER_USAGE_SHADER_DEVICE_ADDRESS_BIT;
 
-    aabb_buffer.init(device, aabb_buffer_size, VK_MEMORY_PROPERTY_HOST_VISIBLE_BIT | VK_MEMORY_PROPERTY_HOST_COHERENT_BIT,
-                     buffer_usage, &alloc_flags);
+    aabb_buffer.init(device, aabb_buffer_size, buffer_usage,
+                     VK_MEMORY_PROPERTY_HOST_VISIBLE_BIT | VK_MEMORY_PROPERTY_HOST_COHERENT_BIT, &alloc_flags);
 
     // Fill buffer with one AABB
     aabb_geometry.SetPrimitiveCount(static_cast<uint32_t>(aabbs.size()));
@@ -676,7 +720,7 @@ GeometryKHR GeometrySimpleOnDeviceAABBInfo(uint32_t vk_api_version, const vk_tes
     return aabb_geometry;
 }
 
-GeometryKHR GeometrySimpleOnHostAABBInfo(uint32_t vk_api_version) {
+GeometryKHR GeometrySimpleOnHostAABBInfo(APIVersion vk_api_version) {
     GeometryKHR aabb_geometry(vk_api_version);
 
     aabb_geometry.SetType(GeometryKHR::Type::AABB);
@@ -693,7 +737,7 @@ GeometryKHR GeometrySimpleOnHostAABBInfo(uint32_t vk_api_version) {
     return aabb_geometry;
 }
 
-GeometryKHR GeometrySimpleDeviceInstance(uint32_t vk_api_version, const vk_testing::Device &device,
+GeometryKHR GeometrySimpleDeviceInstance(APIVersion vk_api_version, const vk_testing::Device &device,
                                          VkAccelerationStructureKHR device_instance) {
     GeometryKHR instance_geometry(vk_api_version);
 
@@ -704,7 +748,7 @@ GeometryKHR GeometrySimpleDeviceInstance(uint32_t vk_api_version, const vk_testi
     return instance_geometry;
 }
 
-GeometryKHR GeometrySimpleHostInstance(uint32_t vk_api_version, VkAccelerationStructureKHR host_instance) {
+GeometryKHR GeometrySimpleHostInstance(APIVersion vk_api_version, VkAccelerationStructureKHR host_instance) {
     GeometryKHR instance_geometry(vk_api_version);
 
     instance_geometry.SetType(GeometryKHR::Type::Instance);
@@ -714,13 +758,13 @@ GeometryKHR GeometrySimpleHostInstance(uint32_t vk_api_version, VkAccelerationSt
     return instance_geometry;
 }
 
-std::shared_ptr<AccelerationStructureKHR> AccelStructNull(uint32_t vk_api_version) {
+std::shared_ptr<AccelerationStructureKHR> AccelStructNull(APIVersion vk_api_version) {
     auto as = std::make_shared<AccelerationStructureKHR>(vk_api_version);
     as->SetNull(true);
     return as;
 }
 
-std::shared_ptr<AccelerationStructureKHR> AccelStructSimpleOnDeviceBottomLevel(uint32_t vk_api_version, VkDeviceSize size) {
+std::shared_ptr<AccelerationStructureKHR> AccelStructSimpleOnDeviceBottomLevel(APIVersion vk_api_version, VkDeviceSize size) {
     auto as = std::make_shared<AccelerationStructureKHR>(vk_api_version);
     as->SetSize(size);
     as->SetType(VK_ACCELERATION_STRUCTURE_TYPE_BOTTOM_LEVEL_KHR);
@@ -732,7 +776,7 @@ std::shared_ptr<AccelerationStructureKHR> AccelStructSimpleOnDeviceBottomLevel(u
     return as;
 }
 
-std::shared_ptr<rt::as::AccelerationStructureKHR> AccelStructSimpleOnHostBottomLevel(uint32_t vk_api_version, VkDeviceSize size) {
+std::shared_ptr<rt::as::AccelerationStructureKHR> AccelStructSimpleOnHostBottomLevel(APIVersion vk_api_version, VkDeviceSize size) {
     auto as = std::make_shared<AccelerationStructureKHR>(vk_api_version);
     as->SetSize(size);
     as->SetType(VK_ACCELERATION_STRUCTURE_TYPE_BOTTOM_LEVEL_KHR);
@@ -743,7 +787,7 @@ std::shared_ptr<rt::as::AccelerationStructureKHR> AccelStructSimpleOnHostBottomL
     return as;
 }
 
-std::shared_ptr<AccelerationStructureKHR> AccelStructSimpleOnDeviceTopLevel(uint32_t vk_api_version, VkDeviceSize size) {
+std::shared_ptr<AccelerationStructureKHR> AccelStructSimpleOnDeviceTopLevel(APIVersion vk_api_version, VkDeviceSize size) {
     auto as = std::make_shared<AccelerationStructureKHR>(vk_api_version);
     as->SetSize(size);
     as->SetType(VK_ACCELERATION_STRUCTURE_TYPE_TOP_LEVEL_KHR);
@@ -755,7 +799,7 @@ std::shared_ptr<AccelerationStructureKHR> AccelStructSimpleOnDeviceTopLevel(uint
     return as;
 }
 
-BuildGeometryInfoKHR BuildGeometryInfoSimpleOnDeviceBottomLevel(uint32_t vk_api_version, const vk_testing::Device &device,
+BuildGeometryInfoKHR BuildGeometryInfoSimpleOnDeviceBottomLevel(APIVersion vk_api_version, const vk_testing::Device &device,
                                                                 GeometryKHR::Type geometry_type /*= GeometryKHR::Type::Triangle*/) {
     BuildGeometryInfoKHR out_build_info(vk_api_version);
 
@@ -791,7 +835,7 @@ BuildGeometryInfoKHR BuildGeometryInfoSimpleOnDeviceBottomLevel(uint32_t vk_api_
     return out_build_info;
 }
 
-BuildGeometryInfoKHR BuildGeometryInfoSimpleOnHostBottomLevel(uint32_t vk_api_version, const vk_testing::Device &device,
+BuildGeometryInfoKHR BuildGeometryInfoSimpleOnHostBottomLevel(APIVersion vk_api_version, const vk_testing::Device &device,
                                                               GeometryKHR::Type geometry_type /*= GeometryKHR::Type::Triangle*/) {
     BuildGeometryInfoKHR out_build_info(vk_api_version);
 
@@ -828,7 +872,7 @@ BuildGeometryInfoKHR BuildGeometryInfoSimpleOnHostBottomLevel(uint32_t vk_api_ve
 }
 
 BuildGeometryInfoKHR BuildGeometryInfoSimpleOnDeviceTopLevel(
-    uint32_t vk_api_version, const vk_testing::Device &device,
+    APIVersion vk_api_version, const vk_testing::Device &device,
     std::shared_ptr<BuildGeometryInfoKHR> on_device_bottom_level_geometry) {
     BuildGeometryInfoKHR out_build_info(vk_api_version);
 
@@ -859,7 +903,7 @@ BuildGeometryInfoKHR BuildGeometryInfoSimpleOnDeviceTopLevel(
     return out_build_info;
 }
 
-BuildGeometryInfoKHR BuildGeometryInfoSimpleOnHostTopLevel(uint32_t vk_api_version, const vk_testing::Device &device,
+BuildGeometryInfoKHR BuildGeometryInfoSimpleOnHostTopLevel(APIVersion vk_api_version, const vk_testing::Device &device,
                                                            std::shared_ptr<BuildGeometryInfoKHR> on_host_bottom_level_geometry) {
     BuildGeometryInfoKHR out_build_info(vk_api_version);
 
