@@ -1,9 +1,6 @@
 /*
- * Copyright (c) 2015-2023 The Khronos Group Inc.
- * Copyright (c) 2015-2023 Valve Corporation
- * Copyright (c) 2015-2023 LunarG, Inc.
- * Copyright (c) 2015-2023 Google, Inc.
- * Modifications Copyright (C) 2020 Advanced Micro Devices, Inc. All rights reserved.
+ * Copyright (c) 2023 Valve Corporation
+ * Copyright (c) 2023 LunarG, Inc.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -57,7 +54,7 @@ class GeometryKHR {
     };
 
     ~GeometryKHR() = default;
-    GeometryKHR(uint32_t vk_api_version);
+    GeometryKHR(APIVersion vk_api_version);
     GeometryKHR(const GeometryKHR&) = delete;
     GeometryKHR(GeometryKHR&&) = default;
     GeometryKHR& operator=(GeometryKHR&&) = default;
@@ -91,7 +88,7 @@ class GeometryKHR {
     const auto& GetAABBs() const { return aabbs_; }
 
   private:
-    uint32_t vk_api_version_;
+    APIVersion vk_api_version_;
     VkAccelerationStructureGeometryKHR vk_obj_;
     Type type_ = Type::_INTERNAL_UNSPECIFIED;
     uint32_t primitiveCount_ = 0;
@@ -103,7 +100,7 @@ class GeometryKHR {
 class AccelerationStructureKHR : public vk_testing::internal::NonDispHandle<VkAccelerationStructureKHR> {
   public:
     ~AccelerationStructureKHR() { Destroy(); }
-    AccelerationStructureKHR(uint32_t vk_api_version);
+    AccelerationStructureKHR(APIVersion vk_api_version);
     AccelerationStructureKHR(AccelerationStructureKHR&& rhs) = default;
     AccelerationStructureKHR& operator=(AccelerationStructureKHR&&) = default;
     AccelerationStructureKHR& operator=(const AccelerationStructureKHR&) = delete;
@@ -132,7 +129,7 @@ class AccelerationStructureKHR : public vk_testing::internal::NonDispHandle<VkAc
 
   private:
     bool is_null_ = false;
-    uint32_t vk_api_version_;
+    APIVersion vk_api_version_;
     VkAccelerationStructureCreateInfoKHR vk_info_;
     vk_testing::Buffer device_buffer_;
     VkMemoryAllocateFlags buffer_memory_allocate_flags_{};
@@ -144,7 +141,7 @@ class AccelerationStructureKHR : public vk_testing::internal::NonDispHandle<VkAc
 class BuildGeometryInfoKHR {
   public:
     ~BuildGeometryInfoKHR() = default;
-    BuildGeometryInfoKHR(uint32_t vk_api_version);
+    BuildGeometryInfoKHR(APIVersion vk_api_version);
     BuildGeometryInfoKHR(BuildGeometryInfoKHR&&) = default;
     BuildGeometryInfoKHR& operator=(BuildGeometryInfoKHR&& rhs) = default;
     BuildGeometryInfoKHR& operator=(const BuildGeometryInfoKHR&) = delete;
@@ -157,17 +154,18 @@ class BuildGeometryInfoKHR {
     // Using the same pointers for src and dst is supported
     BuildGeometryInfoKHR& SetSrcAS(std::shared_ptr<AccelerationStructureKHR> src_as);
     BuildGeometryInfoKHR& SetDstAS(std::shared_ptr<AccelerationStructureKHR> dst_as);
-    BuildGeometryInfoKHR& SetScratchBuffer(vk_testing::Buffer&& scratch_buffer);
+    BuildGeometryInfoKHR& SetScratchBuffer(std::shared_ptr<vk_testing::Buffer> scratch_buffer);
+    BuildGeometryInfoKHR& SetDeviceScratchOffset(VkDeviceAddress offset);
     BuildGeometryInfoKHR& SetBottomLevelAS(std::shared_ptr<BuildGeometryInfoKHR> bottom_level_as);
+    // Should be 0 or 1
     BuildGeometryInfoKHR& SetInfoCount(uint32_t info_count);
     BuildGeometryInfoKHR& SetNullInfos(bool use_null_infos);
     BuildGeometryInfoKHR& SetNullBuildRangeInfos(bool use_null_build_range_infos);
 
     // Those functions call Build() on internal resources (geometries, src and dst acceleration structures, scratch buffer),
     // then one the build acceleration structure function.
-    void BuildCmdBuffer(VkInstance instance, const vk_testing::Device& device, VkCommandBuffer cmd_buffer,
-                        bool use_ppGeometries = true);
-    void BuildCmdBufferIndirect(VkInstance instance, const vk_testing::Device& device, VkCommandBuffer cmd_buffer);
+    void BuildCmdBuffer(const vk_testing::Device& device, VkCommandBuffer cmd_buffer, bool use_ppGeometries = true);
+    void BuildCmdBufferIndirect(const vk_testing::Device& device, VkCommandBuffer cmd_buffer);
     void BuildHost(VkInstance instance, const vk_testing::Device& device);
     void VkCmdBuildAccelerationStructuresKHR(const vk_testing::Device& device, VkCommandBuffer cmd_buffer,
                                              bool use_ppGeometries = true);
@@ -179,25 +177,33 @@ class BuildGeometryInfoKHR {
     auto& GetGeometries() { return geometries_; }
     auto& GetSrcAS() { return src_as_; }
     auto& GetDstAS() { return dst_as_; }
-    auto& GetBottomLevelAS() { return bottom_level_as_; }
+    auto& GetBottomLevelAS() { return blas_; }
     const auto& GetScratchBuffer() const { return device_scratch_; }
     VkAccelerationStructureBuildSizesInfoKHR GetSizeInfo(VkDevice device, bool use_ppGeometries = true);
 
   private:
-    void BuildCommon(VkInstance instance, const vk_testing::Device& device, bool is_on_device_build, bool use_ppGeometries = true);
+    friend void BuildAccelerationStructuresKHR(const vk_testing::Device& device, VkCommandBuffer cmd_buffer,
+                                               std::vector<BuildGeometryInfoKHR>& infos);
 
-    uint32_t vk_api_version_;
+    void BuildCommon(const vk_testing::Device& device, bool is_on_device_build, bool use_ppGeometries = true);
+
+    APIVersion vk_api_version_;
     uint32_t vk_info_count_ = 1;
     bool use_null_infos_ = false;
     bool use_null_build_range_infos_ = false;
     VkAccelerationStructureBuildGeometryInfoKHR vk_info_;
     std::vector<GeometryKHR> geometries_;
     std::shared_ptr<AccelerationStructureKHR> src_as_, dst_as_;
-    vk_testing::Buffer device_scratch_;
+    VkDeviceAddress device_scratch_offset_ = 0;
+    std::shared_ptr<vk_testing::Buffer> device_scratch_;
     std::unique_ptr<uint8_t[]> host_scratch_;
-    std::shared_ptr<BuildGeometryInfoKHR> bottom_level_as_;
+    std::shared_ptr<BuildGeometryInfoKHR> blas_;
 };
 
+// Helper functions
+void BuildAccelerationStructuresKHR(const vk_testing::Device& device, VkCommandBuffer cmd_buffer,
+                                    std::vector<BuildGeometryInfoKHR>& infos);
+
 // Helper functions providing simple, valid objects.
 // Calling Build() on them without further modifications results in a usable and valid Vulkan object.
 // Typical usage probably is:
@@ -213,30 +219,30 @@ class BuildGeometryInfoKHR {
 //    m_commandBuffer->end();
 // }
 namespace blueprint {
-GeometryKHR GeometrySimpleOnDeviceTriangleInfo(uint32_t vk_api_version, const vk_testing::Device& device);
-GeometryKHR GeometrySimpleOnHostTriangleInfo(uint32_t vk_api_version);
-GeometryKHR GeometrySimpleOnDeviceAABBInfo(uint32_t vk_api_version, const vk_testing::Device& device);
-GeometryKHR GeometrySimpleOnHostAABBInfo(uint32_t vk_api_version);
-GeometryKHR GeometrySimpleDeviceInstance(uint32_t vk_api_version, const vk_testing::Device& device,
+GeometryKHR GeometrySimpleOnDeviceTriangleInfo(APIVersion vk_api_version, const vk_testing::Device& device);
+GeometryKHR GeometrySimpleOnHostTriangleInfo(APIVersion vk_api_version);
+GeometryKHR GeometrySimpleOnDeviceAABBInfo(APIVersion vk_api_version, const vk_testing::Device& device);
+GeometryKHR GeometrySimpleOnHostAABBInfo(APIVersion vk_api_version);
+GeometryKHR GeometrySimpleDeviceInstance(APIVersion vk_api_version, const vk_testing::Device& device,
                                          VkAccelerationStructureKHR device_instance);
-GeometryKHR GeometrySimpleHostInstance(uint32_t vk_api_version, VkAccelerationStructureKHR host_instance);
+GeometryKHR GeometrySimpleHostInstance(APIVersion vk_api_version, VkAccelerationStructureKHR host_instance);
 
-std::shared_ptr<AccelerationStructureKHR> AccelStructNull(uint32_t vk_api_version);
-std::shared_ptr<AccelerationStructureKHR> AccelStructSimpleOnDeviceBottomLevel(uint32_t vk_api_version, VkDeviceSize size);
-std::shared_ptr<AccelerationStructureKHR> AccelStructSimpleOnHostBottomLevel(uint32_t vk_api_version, VkDeviceSize size);
-std::shared_ptr<AccelerationStructureKHR> AccelStructSimpleOnDeviceTopLevel(uint32_t vk_api_version, VkDeviceSize size);
+std::shared_ptr<AccelerationStructureKHR> AccelStructNull(APIVersion vk_api_version);
+std::shared_ptr<AccelerationStructureKHR> AccelStructSimpleOnDeviceBottomLevel(APIVersion vk_api_version, VkDeviceSize size);
+std::shared_ptr<AccelerationStructureKHR> AccelStructSimpleOnHostBottomLevel(APIVersion vk_api_version, VkDeviceSize size);
+std::shared_ptr<AccelerationStructureKHR> AccelStructSimpleOnDeviceTopLevel(APIVersion vk_api_version, VkDeviceSize size);
 
-BuildGeometryInfoKHR BuildGeometryInfoSimpleOnDeviceBottomLevel(uint32_t vk_api_version, const vk_testing::Device& device,
+BuildGeometryInfoKHR BuildGeometryInfoSimpleOnDeviceBottomLevel(APIVersion vk_api_version, const vk_testing::Device& device,
                                                                 GeometryKHR::Type geometry_type = GeometryKHR::Type::Triangle);
 
-BuildGeometryInfoKHR BuildGeometryInfoSimpleOnHostBottomLevel(uint32_t vk_api_version, const vk_testing::Device& device,
+BuildGeometryInfoKHR BuildGeometryInfoSimpleOnHostBottomLevel(APIVersion vk_api_version, const vk_testing::Device& device,
                                                               GeometryKHR::Type geometry_type = GeometryKHR::Type::Triangle);
 
 // on_device_bottom_level_geometry must have been built previously, and on the device
-BuildGeometryInfoKHR BuildGeometryInfoSimpleOnDeviceTopLevel(uint32_t vk_api_version, const vk_testing::Device& device,
+BuildGeometryInfoKHR BuildGeometryInfoSimpleOnDeviceTopLevel(APIVersion vk_api_version, const vk_testing::Device& device,
                                                              std::shared_ptr<BuildGeometryInfoKHR> on_device_bottom_level_geometry);
 // on_host_bottom_level_geometry must have been built previously, and on the host
-BuildGeometryInfoKHR BuildGeometryInfoSimpleOnHostTopLevel(uint32_t vk_api_version, const vk_testing::Device& device,
+BuildGeometryInfoKHR BuildGeometryInfoSimpleOnHostTopLevel(APIVersion vk_api_version, const vk_testing::Device& device,
                                                            std::shared_ptr<BuildGeometryInfoKHR> on_host_bottom_level_geometry);
 
 }  // namespace blueprint
